/*---------------------------------------------------------------------------

   Swyx Solutions AG
   Emil-Figge-Str. 86
   44227 Dortmund

  ---------------------------------------------------------------------------

  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
  PURPOSE.

  Copyright (C) 2017 Swyx Solutions AG

  All Rights Reserved.

----------------------------------------------------------------------------*/

#ifndef _CLMGRTYPES_H_
#define _CLMGRTYPES_H_



#if !defined(__midl) && !defined(SW_NONAMESPACE)
namespace CLMgr{
#endif



#define PLAYSOUNDFILE_DEFAULTVOLUME     0xffffffff

#ifndef ENCODELINESTATECHANGED
#define ENCODELINESTATECHANGED(index, state)      ((WORD)((((DWORD_PTR)(index)) & 0x03ff) | ((WORD)(((DWORD_PTR)(state)) & 0x3f)) << 10))
#endif

#ifndef LO10BIT
#define LO10BIT(w)           (((DWORD_PTR)(w)) & 0x3ff)
#endif

#ifndef HI6BIT
#define HI6BIT(w)            ((((DWORD_PTR)(w)) >> 10) & 0x3f)
#endif



//message parameters for state information from line manager to client,
//send as wParam of RegisterWindowMessage(_T("WM_LineMgrNotification")
typedef enum
{
    CLMgrLineStateChangedMessage=0,         //state of at least one line has changed
    CLMgrLineSelectionChangedMessage,       //line in focus has changed
    CLMgrLineDetailsChangedMessage,         //details of at least one line have changed
    CLMgrUserDataChangedMessage,            //user configuration has changed
                                            /*
                                            lParam: what part of the user data has been changed
                                            if lUserDataChangedContext is 0, anything might have changed

                                            typedef enum
                                            {
                                                /// main user record
                                                SCl5UDCCRecord                             = 0x00000001,
                                                /// user's binary data object (blob)
                                                SCl5UDCCBinaryData                         = 0x00000002,
                                                /// user's phonebook
                                                SCl5UDCCPhonebook                          = 0x00000004,
                                                /// user's relations
                                                SCl5UDCCRelations                          = 0x00000008,
                                                /// user's redial list
                                                SCl5UDCCRedialList                         = 0x00000010,
                                                /// user's phone call list
                                                SCl5UDCCPhoneCallList                      = 0x00000020,
                                                /// user's phone callback list
                                                SCl5UDCCPhoneCallbackList                  = 0x00000040,
                                                /// user's situations
                                                SCl5UDCCSituations                         = 0x00000080,
                                            } SCl5UserDataChangeContext;
                                            */   
    CLMgrCallDetailsMessage,                //details of last call are available, post mortem for logging purpose
    CLMgrServerDownMessage,                 //server goes down -> keep interfaces to line manager,
                                            //release all interfaces to PBX, wait for ServerUp message
    CLMgrServerUpMessage,                   //server is up again -> keep interfaces to line manger,
                                            //get new interfaces to PBX
    CLMgrWaveDeviceChanged,                 //speaker / micro has been switched on / off
    CLMgrGroupCallNotificationMessage,      //notification about group call
                                            //lParam=1: notification call available; lParam=0: notification call no longer available
    CLMgrNameKeyStateChangedMessage,        //notification about changed namekey state
    CLMgrNumberOfLinesChangedMessage,       //the number of lines has changed
    CLMgrClientShutDownRequest,             //Client Line Manager requests client to shutdown and release all interfaces
    CLMgrPowerSuspendMessage,               //client machine goes to suspend mode -> keep interfaces to line manager,
                                            //release all interfaces to PBX, ait for PowerResume message
    CLMgrPowerResumeMessage,                //client machine has returned from suspend mode -> keep interfaces to line manager
                                            //get new interfaces to PBX
    CLMgrHandsetStateChangedMessage,        //state of handset has changed
                                            //lParam: new handset state 0 = on hook, 1 = off hook                                      
    CLMgrSkinPhoneCommandMessage,           //for forwarding commands from PlugIn to SwyxIt!.exe.
                                            //lParam will contain the requested command (dwCommandId+dwButtonId)
                                            //should be handled only by SwyxIt!
    CLMgrSkinActionAreaStateChangedMessage, //informing clients about a changed action area state.
                                            //lParam will contain the id of the action area that has changed (dwCommandId+dwButtonId)
                                            //This event will be ignored by SwyxIt!.exe.
    CLMgrSkinInfoDetailChangedMessage,      //informing clients about changed information area details.
                                            //lParam will contain the index of the information detail that has changed.
                                            //This event will be ignored by SwyxIt!.exe.
    CLMgrCallbackOnBusyNotification,        //informing clients about available callback on busy
                                            //lParam=1: callback available; lParam=0: callback no longer available
                                            //lParam=2: callback back on busy was signaled but no user response (pickup / reject)
                                            //lParam=3: there is no "signaled callback on busy" left -> reset button state
    CLMgrAsyncMessage,                      //message for reporting success or problems that occured within state machine / were reported by server
                                            //lParam will contain the result code
    CLMgrCtiPairingStateChanged,            //informing clients about changed state of CTI pairing process
                                            //lParam=new CTI pairing state
                                            //  typedef enum
                                            //  {
                                            //      CtiSessNone=-1,
                                            //      CtiSessIdle=0,                  //no cti activity
                                            //      CtiSessMasterPairingRequested,  //CTI master has requested pairing
                                            //      CtiSessSlavePairingRequested,   //CTI slave has received pairing request
                                            //      CtiSessMasterPaired,            //CTI master has established pairing
                                            //      CtiSessSlavePaired,             //CTI slave has established pairing
                                            //      CtiSessMaxStates
                                            //  } CtiSessionStates;
    CLMgrChatMessage,                       //for call clients that have registered as chat message receiver: new message available
                                            //other clients can ignore this message
    CLMgrChatMessageAck,                    //for call clients that have registered as chat message receiver:
                                            //a chat message sent from this client has been acknowledged by an application on peer side
                                            //lParam=message id of confirmed chat message
    CLMgrRecordingError,                    //Error on recording conversation
                                            //lParam=windows error code
    CLMgrVolumeChanged,                     //Volume of sound device has changed
                                            //lParam=new volume 0..100 percent
    CLMgrAudioModeChanged,                  //Current audio mode has changed
                                            //lParam=new audio mode
                                            //    AudioModeIdle = 0,
                                            //    AudioModeHandset,
                                            //    AudioModeHeadset,
                                            //    AudioModeHandsfreeListening,
                                            //    AudioModeIntercom,
                                            //    AudioModeRinging
    CLMgrMicAdjustLevelMeter,               //OBSOLETE: Peak level meter during microphone adjustment
                                            //          lParam=new level meter value 0..100 percent
    CLMgrMicAdjustProceedMeter,             //OBSOLETE: Proceed level during microphone adjustment
                                            //          lParam=new proceed value 0..100 percent
    CLMgrLineStateChangedMessageEx,         //state of at least line has changed
                                            //lParam: LOBYTE: line index
                                            //        HIBYTE: new state
    CLMgrPlaySoundFileDxProceedMeter,       //Proceed level during PlaySoundFileDx playback
                                            //lParam=new proceed value 0..100 percent
	CLMgrSIPRegistrationStateChanged,	    //registration state of SIP account has changed
                                            //lParam: LOBYTE: Account index
                                            //        HIBYTE: new state
	CLMgrWaveFilePlayed,                    //wave file playback finished
                                            //lParam: line index;
                                            //if -1, the message is related to a LineMgr function PlaySoundFile or PlayToRtp
                                            //if >=0 the message is related to a line function PlaySoundFile of line with this index
	CLMgrFirstDataReceived,                 //wave file playback finished
                                            //lParam: line index;
    CLMgrRegisteredSipDeviceListChanged,    //number of registered dialer mode compatible (SIP) devices for own user changed
                                            //lParam: number of registered devices
    CLMgrDialerStartCallResult,              //message for reporting start call success or failure in dialer mode
                                            //lParam will contain the SIP result code
    CLMgrLineStateChangedMessageEx2,        //state of at least line has changed
                                            //lParam: lower 10 bits: line index (0..1023)
                                            //        upper 6 bits: new state (0..63, currently used 0..15)
    CLMgrMediaEncryptionStatusChanged,      //media encryption status has changed
                                            //lParam: lower 16 bits: line index
                                            //        upper 16 bits: new state
                                            //                         EncryptionStateNoCall = 0,
                                            //                         EncryptionStateNotEncrypted,
                                            //                         EncryptionStateEncrypted
    CLMgrLateDisconnect,                    //signal late disconnect
                                            //lParam: line index
    CLMgrDeviceStateChanged,                //signal changes on available devices
    CLMgrBlockDialStringChanged,            //block dial string changed by CTI partner
    CLMgrPlaySoundFileState,                //lParam: 0 stopped, 1 started, 2 paused
    CLMgrVoicemailPlayerAudioMode,          //lParam: 0 Handset, 1 Headset, 2 Handsfree, 3 OpenListening
    CLMgrPluginNotLicensed,                 //lParam: 0 LotusNotes, 1 VisualContact, 2 Datev, 3 Keylink
	CLMgrCloudConnectorStatus,				//lParam: new state
	CLMgrClientRegisterRequest				//lParam: entity that registered (0 = SwyxIt!, 1 = LyncPlugin)
    
} CLMgrMessages;



//async error messages
typedef enum
{
    CLMgrAsyncNOP=-1,
    CLMgrAsyncErrorCBOnBusyRegisterFailed,   //registration of callback on busy failed
    CLMgrAsyncErrorCBRequestRegisterFailed,  //registration of callback request failed
    CLMgrAsyncCBRequestRegisterSucceeded     //registration of callback request succeeded
} CLMgrAsyncMessages;



//line states
typedef enum
{
    LSNone=-1,
    LSInactive=0,                   //line is inactive
    LSHookOffInternal,              //off hook, internal dialtone
    LSHookOffExternal,              //off hook, external dialtone
    LSRinging,                      //incoming call, ringing
    LSDialing,                      //outgoing call, we are dialing, no sound
    LSAlerting,                     //outgoing call, alerting = ringing on destination
    LSKnocking,                     //outgoing call, knocking = second call ringing on destination
    LSBusy,                         //outgoing call, destination is busy
    LSActive,                       //incoming / outgoing call, logical and physical connection is established
    LSOnHold,                       //incoming / outgoing call, logical connection is established, destination gets music on hold
    LSConferenceActive,             //incoming / outgoing conference, logical and physical connection is established
    LSConferenceOnHold,             //incoming / outgoing conference, logical connection is established, not physcically connected
    LSTerminated,                   //incoming / outgoing connection / call has been disconnected
    LSTransferring,                 //special LSOnHold, call is awaiting to be transferred, peer gets special music on hold
    LSDisabled,                     //special LSInactive, we will temporary not allow calls on that line
    LSDirectCall,                   //incoming call, logical and physical connection is established, micro is muted
    LSWaitingForControlledDev,      //internal state: we are waiting for a connect controlled Device/Number
    LSWaitingForConnect,            //internal state: we are waiting for a connect after IClCall::Accept() or IClCall::Activate()
    LSWaitingForConDc,              //internal state: we are waiting for a connect after IClCall::Accept() or IClCall::Activate()
                                    //but call was direct call, line will go to state  LSDirectCall
    LSWaitingForDiscon,             //internal state: we are waiting for a disconnect after IClCall::Disconnect() ...
    LSWaitingForOnHold,             //internal state: we are waiting for a OnHold event after IClCall::Hold()
    LSWaitingForConfHold,           //internal state: we are waiting for a OnHold event on a conference line
    LSWaitingForConfAct,            //internal state: we are waiting for a connect on a conference line on hold
    LSWaitingForTransfer,           //internal state: active line, we are waiting for transfer
    LSWaitingForDialAck,            //internal state: State between LSHookOffInternal and LSHookOffExternal / LSDialing
    LSWaitingForOnHoldTrans,        //internal state: we are waiting for a OnHold event after IClCall::Hold(), next state should be LSTransferring
    LSActiveAlerting,               //internal state: we are visible connected, call was connected by script but is now alerting again (from servers point of view)
    LSActiveAlertingDC,             //internal state: we are visible connected as Inteccom, call was connected by script but is now alerting again (from servers point of view)
    LSMaxLineStates
} CLMgrLineStates;



//notification sounds for line states
typedef enum
{
    NotifySoundOff=0,                       //no sound
    NotifySoundRingingInternal,             //incoming call, inside PBX, ringing
    NotifySoundRingingExternal,             //incoming call from extern, ringing
    NotifySoundRingingKnocking,             //incoming call, second call = knocking
    NotifySoundDialtoneInternal,            //outgoing call, internal dialtone
    NotifySoundDialtoneExternal,            //outgoing call, external dialtone
    NotifySoundDialtoneAlerting,            //outgoing call, alerting on destination
    NotifySoundDialtoneKnocking,            //outgoing call, knocking = second call on destination
    NotifySoundDialtoneBusy,                //outgoing call, destination busy
    NotifySoundRingingGroupcall,            //incoming call to other group member, group call notification
    NotifySoundLineOnHold,                  //notification about line that is still on hold
    NotifySoundRecordingStart,              //notification to all peers: we started recording
    NotifySoundDialtoneCongestion,          //outgoing call, no line available
    NotifySoundDialtoneSpecialInformation,  //outgoing call, unknown number, ...
    NotifySoundDialtoneMessageWaiting,      //outgoing call, internal dialtone, new voicemail available
    NotifySoundRingingCallbackOnBusy,       //notification about available callback on busy
    NotifySoundRingingSpecial,              //incoming call, special (number depending) ringing sound
    NotifySoundRingingDirectCall,           //notification sound for incoming direct call
    NotifySoundTransferComplete,            //notification sound for completed call transfer
    NotifySoundContinue                     //continue with current sound, do not interrupt
} CLMgrNotificationSounds;



//name key signalling states
typedef enum
{
    NKUnknown=0,
    NKLoggedOut,
    NKLoggedIn,
    NKBusy,
    NKGroupCallNotification,
    NKAway,
    NKDoNotDisturb
} CLMgrNameKeyStates;



//cti session states
typedef enum
{
    CtiSessNone=-1,
    CtiSessIdle=0,                      //no cti activity
    CtiSessMasterWaiting4Ind,           //CTI master has requested pairing, waiting for indications
    CtiSessMasterPairing,               //CTI master has received at least one indication, waiting for select
    CtiSessSlavePairing,                //CTI slave has received pairing request
    CtiSessMasterPaired,                //CTI master has established pairing
    CtiSessSlavePaired,                 //CTI slave has established pairing
    CtiSessMaxStates
} CtiSessionStates;


// CLMgr codec (compression) preferences
typedef enum
{
    CLMgrPreferQualityWithHDAudio = 0,
    CLMgrPreferQuality,
    CLMgrPreferCompression,
    CLMgrAlwaysCompression
} CLMgrCodecPref;


#if !defined(__midl) && !defined(SW_NONAMESPACE)
}; // namespace CLMgr
#endif



#endif //_CLMGRTYPES_H_