using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Xml;
using System.Xml.Serialization;
using System.IO;
using System.Threading;

using CLMGRLib;

namespace IpPbxMPC
{
	/// <summary>
	/// Main application form
	/// </summary>
	public class MainForm : System.Windows.Forms.Form
	{
		/// <summary>
		/// Configuration object. Holds all application configuration parameter
		/// </summary>
		private IpPbxMPCConfig Config;

		/// <summary>
		/// Line manager object of PBX client
		/// </summary>
		private ClientLineMgrClass clmgr;

		/// <summary>
		/// UserId of logged in PBXUser
		/// </summary>
		private System.Int32 UserID;

		/// <summary>
		/// Array of PBX client lines
		/// </summary>
		private CLMGRLib.ClientLine [] Lines;

		/// <summary>
		/// configuration path and filename
		/// </summary>
		private string ConfigFilename;

		/// <summary>
		/// Flag to indicate if media player is currently paused.
		/// This means that we've send the application specific pause command
		/// </summary>
		private bool bPaused;

		/// <summary>
		/// Flag to indicate if we're currently logged into the PBX server
		/// </summary>
		private bool bLoggedIn;

		/// <summary>
		/// Speed dial key definition.
		/// </summary>
		private class SpeedDialKey
		{
			public int    Index;
			public string Name;
			public string Number;

			public override string ToString()
			{
				return Name;
			}
		}

		/// <summary>
		/// Array of all speed dial keys the user has defined.
		/// </summary>
		private SpeedDialKey[] SpeedDialKeys;

		private System.Windows.Forms.ComboBox comboBoxApp;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.Button buttonOptions;
		private System.Windows.Forms.PictureBox pictureBox1;
		private System.Windows.Forms.NotifyIcon notifyIcon1;
		private System.Windows.Forms.GroupBox groupBox1;
		private System.Windows.Forms.RadioButton radioButtonSpeedDialKey;
		private System.Windows.Forms.CheckedListBox checkedListBoxSpeedDials;
		private System.Windows.Forms.RadioButton radioButtonLineState;
		private System.Windows.Forms.Button buttonAdvanced;
		private System.Windows.Forms.StatusBar statusBar1;
		private System.Windows.Forms.StatusBarPanel statusBarPanelState;
		private System.Windows.Forms.StatusBarPanel statusBarPanelServer;
		private System.Windows.Forms.StatusBarPanel statusBarPanelUser;
		private System.Windows.Forms.StatusBarPanel statusBarPanelPaused;
		private System.ComponentModel.IContainer components;

		public MainForm()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();
			
			bPaused = false;
			bLoggedIn = false;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
				// Save configuration data
				SaveConfig();

				// log of from PBX server
				if (clmgr!=null)
				{
					//clmgr.DispReleaseUser(UserID);
					clmgr = null;
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(MainForm));
			this.comboBoxApp = new System.Windows.Forms.ComboBox();
			this.label1 = new System.Windows.Forms.Label();
			this.label2 = new System.Windows.Forms.Label();
			this.buttonOptions = new System.Windows.Forms.Button();
			this.pictureBox1 = new System.Windows.Forms.PictureBox();
			this.notifyIcon1 = new System.Windows.Forms.NotifyIcon(this.components);
			this.groupBox1 = new System.Windows.Forms.GroupBox();
			this.checkedListBoxSpeedDials = new System.Windows.Forms.CheckedListBox();
			this.radioButtonSpeedDialKey = new System.Windows.Forms.RadioButton();
			this.radioButtonLineState = new System.Windows.Forms.RadioButton();
			this.buttonAdvanced = new System.Windows.Forms.Button();
			this.statusBar1 = new System.Windows.Forms.StatusBar();
			this.statusBarPanelPaused = new System.Windows.Forms.StatusBarPanel();
			this.statusBarPanelState = new System.Windows.Forms.StatusBarPanel();
			this.statusBarPanelServer = new System.Windows.Forms.StatusBarPanel();
			this.statusBarPanelUser = new System.Windows.Forms.StatusBarPanel();
			this.groupBox1.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.statusBarPanelPaused)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.statusBarPanelState)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.statusBarPanelServer)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.statusBarPanelUser)).BeginInit();
			this.SuspendLayout();
			// 
			// comboBoxApp
			// 
			this.comboBoxApp.AccessibleDescription = resources.GetString("comboBoxApp.AccessibleDescription");
			this.comboBoxApp.AccessibleName = resources.GetString("comboBoxApp.AccessibleName");
			this.comboBoxApp.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("comboBoxApp.Anchor")));
			this.comboBoxApp.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("comboBoxApp.BackgroundImage")));
			this.comboBoxApp.DisplayMember = "ControlledApp";
			this.comboBoxApp.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("comboBoxApp.Dock")));
			this.comboBoxApp.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.comboBoxApp.DropDownWidth = 224;
			this.comboBoxApp.Enabled = ((bool)(resources.GetObject("comboBoxApp.Enabled")));
			this.comboBoxApp.Font = ((System.Drawing.Font)(resources.GetObject("comboBoxApp.Font")));
			this.comboBoxApp.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("comboBoxApp.ImeMode")));
			this.comboBoxApp.IntegralHeight = ((bool)(resources.GetObject("comboBoxApp.IntegralHeight")));
			this.comboBoxApp.ItemHeight = ((int)(resources.GetObject("comboBoxApp.ItemHeight")));
			this.comboBoxApp.Items.AddRange(new object[] {
															 resources.GetString("comboBoxApp.Items"),
															 resources.GetString("comboBoxApp.Items1"),
															 resources.GetString("comboBoxApp.Items2")});
			this.comboBoxApp.Location = ((System.Drawing.Point)(resources.GetObject("comboBoxApp.Location")));
			this.comboBoxApp.MaxDropDownItems = ((int)(resources.GetObject("comboBoxApp.MaxDropDownItems")));
			this.comboBoxApp.MaxLength = ((int)(resources.GetObject("comboBoxApp.MaxLength")));
			this.comboBoxApp.Name = "comboBoxApp";
			this.comboBoxApp.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("comboBoxApp.RightToLeft")));
			this.comboBoxApp.Size = ((System.Drawing.Size)(resources.GetObject("comboBoxApp.Size")));
			this.comboBoxApp.TabIndex = ((int)(resources.GetObject("comboBoxApp.TabIndex")));
			this.comboBoxApp.Text = resources.GetString("comboBoxApp.Text");
			this.comboBoxApp.ValueMember = "ControlledApp";
			this.comboBoxApp.Visible = ((bool)(resources.GetObject("comboBoxApp.Visible")));
			this.comboBoxApp.SelectedIndexChanged += new System.EventHandler(this.comboBoxApp_SelectedIndexChanged);
			// 
			// label1
			// 
			this.label1.AccessibleDescription = resources.GetString("label1.AccessibleDescription");
			this.label1.AccessibleName = resources.GetString("label1.AccessibleName");
			this.label1.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("label1.Anchor")));
			this.label1.AutoSize = ((bool)(resources.GetObject("label1.AutoSize")));
			this.label1.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("label1.Dock")));
			this.label1.Enabled = ((bool)(resources.GetObject("label1.Enabled")));
			this.label1.Font = ((System.Drawing.Font)(resources.GetObject("label1.Font")));
			this.label1.Image = ((System.Drawing.Image)(resources.GetObject("label1.Image")));
			this.label1.ImageAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("label1.ImageAlign")));
			this.label1.ImageIndex = ((int)(resources.GetObject("label1.ImageIndex")));
			this.label1.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("label1.ImeMode")));
			this.label1.Location = ((System.Drawing.Point)(resources.GetObject("label1.Location")));
			this.label1.Name = "label1";
			this.label1.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("label1.RightToLeft")));
			this.label1.Size = ((System.Drawing.Size)(resources.GetObject("label1.Size")));
			this.label1.TabIndex = ((int)(resources.GetObject("label1.TabIndex")));
			this.label1.Text = resources.GetString("label1.Text");
			this.label1.TextAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("label1.TextAlign")));
			this.label1.Visible = ((bool)(resources.GetObject("label1.Visible")));
			// 
			// label2
			// 
			this.label2.AccessibleDescription = resources.GetString("label2.AccessibleDescription");
			this.label2.AccessibleName = resources.GetString("label2.AccessibleName");
			this.label2.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("label2.Anchor")));
			this.label2.AutoSize = ((bool)(resources.GetObject("label2.AutoSize")));
			this.label2.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("label2.Dock")));
			this.label2.Enabled = ((bool)(resources.GetObject("label2.Enabled")));
			this.label2.Font = ((System.Drawing.Font)(resources.GetObject("label2.Font")));
			this.label2.Image = ((System.Drawing.Image)(resources.GetObject("label2.Image")));
			this.label2.ImageAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("label2.ImageAlign")));
			this.label2.ImageIndex = ((int)(resources.GetObject("label2.ImageIndex")));
			this.label2.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("label2.ImeMode")));
			this.label2.Location = ((System.Drawing.Point)(resources.GetObject("label2.Location")));
			this.label2.Name = "label2";
			this.label2.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("label2.RightToLeft")));
			this.label2.Size = ((System.Drawing.Size)(resources.GetObject("label2.Size")));
			this.label2.TabIndex = ((int)(resources.GetObject("label2.TabIndex")));
			this.label2.Text = resources.GetString("label2.Text");
			this.label2.TextAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("label2.TextAlign")));
			this.label2.Visible = ((bool)(resources.GetObject("label2.Visible")));
			// 
			// buttonOptions
			// 
			this.buttonOptions.AccessibleDescription = resources.GetString("buttonOptions.AccessibleDescription");
			this.buttonOptions.AccessibleName = resources.GetString("buttonOptions.AccessibleName");
			this.buttonOptions.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("buttonOptions.Anchor")));
			this.buttonOptions.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("buttonOptions.BackgroundImage")));
			this.buttonOptions.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("buttonOptions.Dock")));
			this.buttonOptions.Enabled = ((bool)(resources.GetObject("buttonOptions.Enabled")));
			this.buttonOptions.FlatStyle = ((System.Windows.Forms.FlatStyle)(resources.GetObject("buttonOptions.FlatStyle")));
			this.buttonOptions.Font = ((System.Drawing.Font)(resources.GetObject("buttonOptions.Font")));
			this.buttonOptions.Image = ((System.Drawing.Image)(resources.GetObject("buttonOptions.Image")));
			this.buttonOptions.ImageAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("buttonOptions.ImageAlign")));
			this.buttonOptions.ImageIndex = ((int)(resources.GetObject("buttonOptions.ImageIndex")));
			this.buttonOptions.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("buttonOptions.ImeMode")));
			this.buttonOptions.Location = ((System.Drawing.Point)(resources.GetObject("buttonOptions.Location")));
			this.buttonOptions.Name = "buttonOptions";
			this.buttonOptions.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("buttonOptions.RightToLeft")));
			this.buttonOptions.Size = ((System.Drawing.Size)(resources.GetObject("buttonOptions.Size")));
			this.buttonOptions.TabIndex = ((int)(resources.GetObject("buttonOptions.TabIndex")));
			this.buttonOptions.Text = resources.GetString("buttonOptions.Text");
			this.buttonOptions.TextAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("buttonOptions.TextAlign")));
			this.buttonOptions.Visible = ((bool)(resources.GetObject("buttonOptions.Visible")));
			this.buttonOptions.Click += new System.EventHandler(this.buttonOptions_Click);
			// 
			// pictureBox1
			// 
			this.pictureBox1.AccessibleDescription = resources.GetString("pictureBox1.AccessibleDescription");
			this.pictureBox1.AccessibleName = resources.GetString("pictureBox1.AccessibleName");
			this.pictureBox1.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("pictureBox1.Anchor")));
			this.pictureBox1.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("pictureBox1.BackgroundImage")));
			this.pictureBox1.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("pictureBox1.Dock")));
			this.pictureBox1.Enabled = ((bool)(resources.GetObject("pictureBox1.Enabled")));
			this.pictureBox1.Font = ((System.Drawing.Font)(resources.GetObject("pictureBox1.Font")));
			this.pictureBox1.Image = ((System.Drawing.Image)(resources.GetObject("pictureBox1.Image")));
			this.pictureBox1.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("pictureBox1.ImeMode")));
			this.pictureBox1.Location = ((System.Drawing.Point)(resources.GetObject("pictureBox1.Location")));
			this.pictureBox1.Name = "pictureBox1";
			this.pictureBox1.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("pictureBox1.RightToLeft")));
			this.pictureBox1.Size = ((System.Drawing.Size)(resources.GetObject("pictureBox1.Size")));
			this.pictureBox1.SizeMode = ((System.Windows.Forms.PictureBoxSizeMode)(resources.GetObject("pictureBox1.SizeMode")));
			this.pictureBox1.TabIndex = ((int)(resources.GetObject("pictureBox1.TabIndex")));
			this.pictureBox1.TabStop = false;
			this.pictureBox1.Text = resources.GetString("pictureBox1.Text");
			this.pictureBox1.Visible = ((bool)(resources.GetObject("pictureBox1.Visible")));
			this.pictureBox1.Click += new System.EventHandler(this.pictureBox1_Click);
			// 
			// notifyIcon1
			// 
			this.notifyIcon1.Icon = ((System.Drawing.Icon)(resources.GetObject("notifyIcon1.Icon")));
			this.notifyIcon1.Text = resources.GetString("notifyIcon1.Text");
			this.notifyIcon1.Visible = ((bool)(resources.GetObject("notifyIcon1.Visible")));
			this.notifyIcon1.DoubleClick += new System.EventHandler(this.notifyIcon1_Click);
			// 
			// groupBox1
			// 
			this.groupBox1.AccessibleDescription = resources.GetString("groupBox1.AccessibleDescription");
			this.groupBox1.AccessibleName = resources.GetString("groupBox1.AccessibleName");
			this.groupBox1.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("groupBox1.Anchor")));
			this.groupBox1.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("groupBox1.BackgroundImage")));
			this.groupBox1.Controls.Add(this.checkedListBoxSpeedDials);
			this.groupBox1.Controls.Add(this.radioButtonSpeedDialKey);
			this.groupBox1.Controls.Add(this.radioButtonLineState);
			this.groupBox1.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("groupBox1.Dock")));
			this.groupBox1.Enabled = ((bool)(resources.GetObject("groupBox1.Enabled")));
			this.groupBox1.Font = ((System.Drawing.Font)(resources.GetObject("groupBox1.Font")));
			this.groupBox1.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("groupBox1.ImeMode")));
			this.groupBox1.Location = ((System.Drawing.Point)(resources.GetObject("groupBox1.Location")));
			this.groupBox1.Name = "groupBox1";
			this.groupBox1.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("groupBox1.RightToLeft")));
			this.groupBox1.Size = ((System.Drawing.Size)(resources.GetObject("groupBox1.Size")));
			this.groupBox1.TabIndex = ((int)(resources.GetObject("groupBox1.TabIndex")));
			this.groupBox1.TabStop = false;
			this.groupBox1.Text = resources.GetString("groupBox1.Text");
			this.groupBox1.Visible = ((bool)(resources.GetObject("groupBox1.Visible")));
			// 
			// checkedListBoxSpeedDials
			// 
			this.checkedListBoxSpeedDials.AccessibleDescription = resources.GetString("checkedListBoxSpeedDials.AccessibleDescription");
			this.checkedListBoxSpeedDials.AccessibleName = resources.GetString("checkedListBoxSpeedDials.AccessibleName");
			this.checkedListBoxSpeedDials.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("checkedListBoxSpeedDials.Anchor")));
			this.checkedListBoxSpeedDials.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("checkedListBoxSpeedDials.BackgroundImage")));
			this.checkedListBoxSpeedDials.ColumnWidth = ((int)(resources.GetObject("checkedListBoxSpeedDials.ColumnWidth")));
			this.checkedListBoxSpeedDials.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("checkedListBoxSpeedDials.Dock")));
			this.checkedListBoxSpeedDials.Enabled = ((bool)(resources.GetObject("checkedListBoxSpeedDials.Enabled")));
			this.checkedListBoxSpeedDials.Font = ((System.Drawing.Font)(resources.GetObject("checkedListBoxSpeedDials.Font")));
			this.checkedListBoxSpeedDials.HorizontalExtent = ((int)(resources.GetObject("checkedListBoxSpeedDials.HorizontalExtent")));
			this.checkedListBoxSpeedDials.HorizontalScrollbar = ((bool)(resources.GetObject("checkedListBoxSpeedDials.HorizontalScrollbar")));
			this.checkedListBoxSpeedDials.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("checkedListBoxSpeedDials.ImeMode")));
			this.checkedListBoxSpeedDials.IntegralHeight = ((bool)(resources.GetObject("checkedListBoxSpeedDials.IntegralHeight")));
			this.checkedListBoxSpeedDials.Location = ((System.Drawing.Point)(resources.GetObject("checkedListBoxSpeedDials.Location")));
			this.checkedListBoxSpeedDials.Name = "checkedListBoxSpeedDials";
			this.checkedListBoxSpeedDials.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("checkedListBoxSpeedDials.RightToLeft")));
			this.checkedListBoxSpeedDials.ScrollAlwaysVisible = ((bool)(resources.GetObject("checkedListBoxSpeedDials.ScrollAlwaysVisible")));
			this.checkedListBoxSpeedDials.Size = ((System.Drawing.Size)(resources.GetObject("checkedListBoxSpeedDials.Size")));
			this.checkedListBoxSpeedDials.TabIndex = ((int)(resources.GetObject("checkedListBoxSpeedDials.TabIndex")));
			this.checkedListBoxSpeedDials.Visible = ((bool)(resources.GetObject("checkedListBoxSpeedDials.Visible")));
			this.checkedListBoxSpeedDials.ItemCheck += new System.Windows.Forms.ItemCheckEventHandler(this.checkedListBoxSpeedDials_ItemCheck);
			// 
			// radioButtonSpeedDialKey
			// 
			this.radioButtonSpeedDialKey.AccessibleDescription = resources.GetString("radioButtonSpeedDialKey.AccessibleDescription");
			this.radioButtonSpeedDialKey.AccessibleName = resources.GetString("radioButtonSpeedDialKey.AccessibleName");
			this.radioButtonSpeedDialKey.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("radioButtonSpeedDialKey.Anchor")));
			this.radioButtonSpeedDialKey.Appearance = ((System.Windows.Forms.Appearance)(resources.GetObject("radioButtonSpeedDialKey.Appearance")));
			this.radioButtonSpeedDialKey.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("radioButtonSpeedDialKey.BackgroundImage")));
			this.radioButtonSpeedDialKey.CheckAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("radioButtonSpeedDialKey.CheckAlign")));
			this.radioButtonSpeedDialKey.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("radioButtonSpeedDialKey.Dock")));
			this.radioButtonSpeedDialKey.Enabled = ((bool)(resources.GetObject("radioButtonSpeedDialKey.Enabled")));
			this.radioButtonSpeedDialKey.FlatStyle = ((System.Windows.Forms.FlatStyle)(resources.GetObject("radioButtonSpeedDialKey.FlatStyle")));
			this.radioButtonSpeedDialKey.Font = ((System.Drawing.Font)(resources.GetObject("radioButtonSpeedDialKey.Font")));
			this.radioButtonSpeedDialKey.Image = ((System.Drawing.Image)(resources.GetObject("radioButtonSpeedDialKey.Image")));
			this.radioButtonSpeedDialKey.ImageAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("radioButtonSpeedDialKey.ImageAlign")));
			this.radioButtonSpeedDialKey.ImageIndex = ((int)(resources.GetObject("radioButtonSpeedDialKey.ImageIndex")));
			this.radioButtonSpeedDialKey.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("radioButtonSpeedDialKey.ImeMode")));
			this.radioButtonSpeedDialKey.Location = ((System.Drawing.Point)(resources.GetObject("radioButtonSpeedDialKey.Location")));
			this.radioButtonSpeedDialKey.Name = "radioButtonSpeedDialKey";
			this.radioButtonSpeedDialKey.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("radioButtonSpeedDialKey.RightToLeft")));
			this.radioButtonSpeedDialKey.Size = ((System.Drawing.Size)(resources.GetObject("radioButtonSpeedDialKey.Size")));
			this.radioButtonSpeedDialKey.TabIndex = ((int)(resources.GetObject("radioButtonSpeedDialKey.TabIndex")));
			this.radioButtonSpeedDialKey.Text = resources.GetString("radioButtonSpeedDialKey.Text");
			this.radioButtonSpeedDialKey.TextAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("radioButtonSpeedDialKey.TextAlign")));
			this.radioButtonSpeedDialKey.Visible = ((bool)(resources.GetObject("radioButtonSpeedDialKey.Visible")));
			this.radioButtonSpeedDialKey.CheckedChanged += new System.EventHandler(this.radioButtonSpeedDialKey_CheckedChanged);
			// 
			// radioButtonLineState
			// 
			this.radioButtonLineState.AccessibleDescription = resources.GetString("radioButtonLineState.AccessibleDescription");
			this.radioButtonLineState.AccessibleName = resources.GetString("radioButtonLineState.AccessibleName");
			this.radioButtonLineState.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("radioButtonLineState.Anchor")));
			this.radioButtonLineState.Appearance = ((System.Windows.Forms.Appearance)(resources.GetObject("radioButtonLineState.Appearance")));
			this.radioButtonLineState.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("radioButtonLineState.BackgroundImage")));
			this.radioButtonLineState.CheckAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("radioButtonLineState.CheckAlign")));
			this.radioButtonLineState.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("radioButtonLineState.Dock")));
			this.radioButtonLineState.Enabled = ((bool)(resources.GetObject("radioButtonLineState.Enabled")));
			this.radioButtonLineState.FlatStyle = ((System.Windows.Forms.FlatStyle)(resources.GetObject("radioButtonLineState.FlatStyle")));
			this.radioButtonLineState.Font = ((System.Drawing.Font)(resources.GetObject("radioButtonLineState.Font")));
			this.radioButtonLineState.Image = ((System.Drawing.Image)(resources.GetObject("radioButtonLineState.Image")));
			this.radioButtonLineState.ImageAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("radioButtonLineState.ImageAlign")));
			this.radioButtonLineState.ImageIndex = ((int)(resources.GetObject("radioButtonLineState.ImageIndex")));
			this.radioButtonLineState.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("radioButtonLineState.ImeMode")));
			this.radioButtonLineState.Location = ((System.Drawing.Point)(resources.GetObject("radioButtonLineState.Location")));
			this.radioButtonLineState.Name = "radioButtonLineState";
			this.radioButtonLineState.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("radioButtonLineState.RightToLeft")));
			this.radioButtonLineState.Size = ((System.Drawing.Size)(resources.GetObject("radioButtonLineState.Size")));
			this.radioButtonLineState.TabIndex = ((int)(resources.GetObject("radioButtonLineState.TabIndex")));
			this.radioButtonLineState.Text = resources.GetString("radioButtonLineState.Text");
			this.radioButtonLineState.TextAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("radioButtonLineState.TextAlign")));
			this.radioButtonLineState.Visible = ((bool)(resources.GetObject("radioButtonLineState.Visible")));
			// 
			// buttonAdvanced
			// 
			this.buttonAdvanced.AccessibleDescription = resources.GetString("buttonAdvanced.AccessibleDescription");
			this.buttonAdvanced.AccessibleName = resources.GetString("buttonAdvanced.AccessibleName");
			this.buttonAdvanced.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("buttonAdvanced.Anchor")));
			this.buttonAdvanced.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("buttonAdvanced.BackgroundImage")));
			this.buttonAdvanced.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("buttonAdvanced.Dock")));
			this.buttonAdvanced.Enabled = ((bool)(resources.GetObject("buttonAdvanced.Enabled")));
			this.buttonAdvanced.FlatStyle = ((System.Windows.Forms.FlatStyle)(resources.GetObject("buttonAdvanced.FlatStyle")));
			this.buttonAdvanced.Font = ((System.Drawing.Font)(resources.GetObject("buttonAdvanced.Font")));
			this.buttonAdvanced.Image = ((System.Drawing.Image)(resources.GetObject("buttonAdvanced.Image")));
			this.buttonAdvanced.ImageAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("buttonAdvanced.ImageAlign")));
			this.buttonAdvanced.ImageIndex = ((int)(resources.GetObject("buttonAdvanced.ImageIndex")));
			this.buttonAdvanced.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("buttonAdvanced.ImeMode")));
			this.buttonAdvanced.Location = ((System.Drawing.Point)(resources.GetObject("buttonAdvanced.Location")));
			this.buttonAdvanced.Name = "buttonAdvanced";
			this.buttonAdvanced.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("buttonAdvanced.RightToLeft")));
			this.buttonAdvanced.Size = ((System.Drawing.Size)(resources.GetObject("buttonAdvanced.Size")));
			this.buttonAdvanced.TabIndex = ((int)(resources.GetObject("buttonAdvanced.TabIndex")));
			this.buttonAdvanced.Text = resources.GetString("buttonAdvanced.Text");
			this.buttonAdvanced.TextAlign = ((System.Drawing.ContentAlignment)(resources.GetObject("buttonAdvanced.TextAlign")));
			this.buttonAdvanced.Visible = ((bool)(resources.GetObject("buttonAdvanced.Visible")));
			this.buttonAdvanced.Click += new System.EventHandler(this.buttonAdvanced_Click);
			// 
			// statusBar1
			// 
			this.statusBar1.AccessibleDescription = resources.GetString("statusBar1.AccessibleDescription");
			this.statusBar1.AccessibleName = resources.GetString("statusBar1.AccessibleName");
			this.statusBar1.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("statusBar1.Anchor")));
			this.statusBar1.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("statusBar1.BackgroundImage")));
			this.statusBar1.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("statusBar1.Dock")));
			this.statusBar1.Enabled = ((bool)(resources.GetObject("statusBar1.Enabled")));
			this.statusBar1.Font = ((System.Drawing.Font)(resources.GetObject("statusBar1.Font")));
			this.statusBar1.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("statusBar1.ImeMode")));
			this.statusBar1.Location = ((System.Drawing.Point)(resources.GetObject("statusBar1.Location")));
			this.statusBar1.Name = "statusBar1";
			this.statusBar1.Panels.AddRange(new System.Windows.Forms.StatusBarPanel[] {
																						  this.statusBarPanelPaused,
																						  this.statusBarPanelState,
																						  this.statusBarPanelServer,
																						  this.statusBarPanelUser});
			this.statusBar1.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("statusBar1.RightToLeft")));
			this.statusBar1.ShowPanels = true;
			this.statusBar1.Size = ((System.Drawing.Size)(resources.GetObject("statusBar1.Size")));
			this.statusBar1.SizingGrip = false;
			this.statusBar1.TabIndex = ((int)(resources.GetObject("statusBar1.TabIndex")));
			this.statusBar1.Text = resources.GetString("statusBar1.Text");
			this.statusBar1.Visible = ((bool)(resources.GetObject("statusBar1.Visible")));
			// 
			// statusBarPanelPaused
			// 
			this.statusBarPanelPaused.Alignment = ((System.Windows.Forms.HorizontalAlignment)(resources.GetObject("statusBarPanelPaused.Alignment")));
			this.statusBarPanelPaused.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Spring;
			this.statusBarPanelPaused.Icon = ((System.Drawing.Icon)(resources.GetObject("statusBarPanelPaused.Icon")));
			this.statusBarPanelPaused.MinWidth = ((int)(resources.GetObject("statusBarPanelPaused.MinWidth")));
			this.statusBarPanelPaused.Text = resources.GetString("statusBarPanelPaused.Text");
			this.statusBarPanelPaused.ToolTipText = resources.GetString("statusBarPanelPaused.ToolTipText");
			this.statusBarPanelPaused.Width = ((int)(resources.GetObject("statusBarPanelPaused.Width")));
			// 
			// statusBarPanelState
			// 
			this.statusBarPanelState.Alignment = ((System.Windows.Forms.HorizontalAlignment)(resources.GetObject("statusBarPanelState.Alignment")));
			this.statusBarPanelState.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Contents;
			this.statusBarPanelState.Icon = ((System.Drawing.Icon)(resources.GetObject("statusBarPanelState.Icon")));
			this.statusBarPanelState.MinWidth = ((int)(resources.GetObject("statusBarPanelState.MinWidth")));
			this.statusBarPanelState.Text = resources.GetString("statusBarPanelState.Text");
			this.statusBarPanelState.ToolTipText = resources.GetString("statusBarPanelState.ToolTipText");
			this.statusBarPanelState.Width = ((int)(resources.GetObject("statusBarPanelState.Width")));
			// 
			// statusBarPanelServer
			// 
			this.statusBarPanelServer.Alignment = ((System.Windows.Forms.HorizontalAlignment)(resources.GetObject("statusBarPanelServer.Alignment")));
			this.statusBarPanelServer.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Contents;
			this.statusBarPanelServer.Icon = ((System.Drawing.Icon)(resources.GetObject("statusBarPanelServer.Icon")));
			this.statusBarPanelServer.MinWidth = ((int)(resources.GetObject("statusBarPanelServer.MinWidth")));
			this.statusBarPanelServer.Text = resources.GetString("statusBarPanelServer.Text");
			this.statusBarPanelServer.ToolTipText = resources.GetString("statusBarPanelServer.ToolTipText");
			this.statusBarPanelServer.Width = ((int)(resources.GetObject("statusBarPanelServer.Width")));
			// 
			// statusBarPanelUser
			// 
			this.statusBarPanelUser.Alignment = ((System.Windows.Forms.HorizontalAlignment)(resources.GetObject("statusBarPanelUser.Alignment")));
			this.statusBarPanelUser.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Contents;
			this.statusBarPanelUser.Icon = ((System.Drawing.Icon)(resources.GetObject("statusBarPanelUser.Icon")));
			this.statusBarPanelUser.MinWidth = ((int)(resources.GetObject("statusBarPanelUser.MinWidth")));
			this.statusBarPanelUser.Text = resources.GetString("statusBarPanelUser.Text");
			this.statusBarPanelUser.ToolTipText = resources.GetString("statusBarPanelUser.ToolTipText");
			this.statusBarPanelUser.Width = ((int)(resources.GetObject("statusBarPanelUser.Width")));
			// 
			// MainForm
			// 
			this.AccessibleDescription = resources.GetString("$this.AccessibleDescription");
			this.AccessibleName = resources.GetString("$this.AccessibleName");
			this.AutoScaleBaseSize = ((System.Drawing.Size)(resources.GetObject("$this.AutoScaleBaseSize")));
			this.AutoScroll = ((bool)(resources.GetObject("$this.AutoScroll")));
			this.AutoScrollMargin = ((System.Drawing.Size)(resources.GetObject("$this.AutoScrollMargin")));
			this.AutoScrollMinSize = ((System.Drawing.Size)(resources.GetObject("$this.AutoScrollMinSize")));
			this.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("$this.BackgroundImage")));
			this.ClientSize = ((System.Drawing.Size)(resources.GetObject("$this.ClientSize")));
			this.Controls.Add(this.statusBar1);
			this.Controls.Add(this.buttonAdvanced);
			this.Controls.Add(this.groupBox1);
			this.Controls.Add(this.pictureBox1);
			this.Controls.Add(this.buttonOptions);
			this.Controls.Add(this.label2);
			this.Controls.Add(this.label1);
			this.Controls.Add(this.comboBoxApp);
			this.Enabled = ((bool)(resources.GetObject("$this.Enabled")));
			this.Font = ((System.Drawing.Font)(resources.GetObject("$this.Font")));
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("$this.ImeMode")));
			this.Location = ((System.Drawing.Point)(resources.GetObject("$this.Location")));
			this.MaximizeBox = false;
			this.MaximumSize = ((System.Drawing.Size)(resources.GetObject("$this.MaximumSize")));
			this.MinimumSize = ((System.Drawing.Size)(resources.GetObject("$this.MinimumSize")));
			this.Name = "MainForm";
			this.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("$this.RightToLeft")));
			this.StartPosition = ((System.Windows.Forms.FormStartPosition)(resources.GetObject("$this.StartPosition")));
			this.Text = resources.GetString("$this.Text");
			this.Resize += new System.EventHandler(this.MainForm_Resize);
			this.Closing += new System.ComponentModel.CancelEventHandler(this.MainForm_Closing);
			this.Load += new System.EventHandler(this.MainForm_Load);
			this.groupBox1.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.statusBarPanelPaused)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.statusBarPanelState)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.statusBarPanelServer)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.statusBarPanelUser)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new MainForm());
		}

		/// <summary>
		/// Load configuration data to deserialize config file to a IpPbxMPCConfig object.
		/// </summary>
		private void LoadConfig()
		{
			try
			{
				if (System.IO.File.Exists(ConfigFilename))
				{
					XmlSerializer serializer = new XmlSerializer(typeof(IpPbxMPCConfig));
					/* If the XML document has been altered with unknown 
					nodes or attributes, handle them with the 
					UnknownNode and UnknownAttribute events.*/
					serializer.UnknownNode+= new 
						XmlNodeEventHandler(serializer_UnknownNode);
					serializer.UnknownAttribute+= new 
						XmlAttributeEventHandler(serializer_UnknownAttribute);

					// A FileStream is needed to read the XML document.
					FileStream fs = new FileStream(ConfigFilename, FileMode.Open);

					Config = (IpPbxMPCConfig) serializer.Deserialize(fs);
					fs.Close();
				}
				else
				{
					// no config file found, create default config
					Config = new IpPbxMPCConfig();
					Config.ControlledApplication = IpPbxMPCConfig.ControlledApp.Winamp;
					Config.Apps= new IpPbxApp[3];
					Config.Apps[0] = new IpPbxGeneric();
					Config.Apps[1] = new IpPbxWinamp();
					Config.Apps[2] = new IpPbxBrowseAmp();
					IpPbxBrowseAmp ba = (IpPbxBrowseAmp)Config.Apps[2];
					ba.PauseURL = "http://localhost:888/index.html?pause";
					ba.UnPauseURL = "http://localhost:888/index.html?playifnotplaying";
				}
			} 
			catch(System.Exception e)
			{
				Console.WriteLine(e);
				if (e.InnerException!=null)
					Console.WriteLine(e.InnerException);
			}
		}

		protected void serializer_UnknownNode
			(object sender, XmlNodeEventArgs e)
		{
			Console.WriteLine("Unknown Node:" +   e.Name + "\t" + e.Text);
		}

		protected void serializer_UnknownAttribute
			(object sender, XmlAttributeEventArgs e)
		{
			System.Xml.XmlAttribute attr = e.Attr;
			Console.WriteLine("Unknown attribute " + 
				attr.Name + "='" + attr.Value + "'");
		}

		/// <summary>
		/// save configuration data by serializing the IpPbxMPCConfig object to a
		/// XML file
		/// </summary>
		private void SaveConfig()
		{
			try
			{
				// Create an instance of the XmlSerializer class;
				// specify the type of object to serialize.
				XmlSerializer serializer = 
					new XmlSerializer(typeof(IpPbxMPCConfig));
				TextWriter writer = new StreamWriter(ConfigFilename);
				serializer.Serialize(writer, Config);
				writer.Close();
			}
			catch (System.Exception e)
			{
				System.Console.WriteLine(e);
			}
		}

		private void buttonClose_Click(object sender, System.EventArgs e)
		{
			this.Close();
		}

		private void comboBoxApp_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			Config.ControlledApplication = (IpPbxMPCConfig.ControlledApp)comboBoxApp.SelectedIndex;
			buttonOptions.Enabled = Config[Config.ControlledApplication].HasOptions();
		}
		
		/// <summary>
		/// Line manager notification method. PBX client line manager calls this method
		/// to deliver events
		/// </summary>
		/// <param name="msg">type of event.</param>
		/// <param name="param">event parameter</param>
		public void OnLineMgrNotification(System.Int32 msg, System.Int32 param)
		{
			// Note: There's no enum for message types in the typelib, therefore we
			// have to use integer directly
			switch (msg)
			{
				case 0: // line state change event. 
				{
					// if we're configured to react on line states, call the appropriate 
					// method
					if (Config.EventToReactOn==IpPbxMPCConfig.IpPbxEvents.NonActiveLineKey)
					{
						int LineIdx = clmgr.DispSelectedLineNumber;
						OnLineStateChanged(LineIdx);
					}
					break;
				}
				case 9:
				{
					// if we're configured to react on speed dial key states, call the appropriate 
					// method
					if (Config.EventToReactOn==IpPbxMPCConfig.IpPbxEvents.SpeedDialKeyState)
					{
						OnSpeedDialKeyChanged(param);
					}
					break;
				}
			}
		}

		/// <summary>
		/// Called when one of the monitored speed dial key has changed its state
		/// </summary>
		/// <param name="NameKeyIndex">Index of the speed dial key which changed the state</param>
		public void OnSpeedDialKeyChanged(int NameKeyIndex)
		{
			if (   NameKeyIndex>=SpeedDialKeys.GetLowerBound(0)
				&& NameKeyIndex<=SpeedDialKeys.GetUpperBound(0))
			{
				SpeedDialKey k = SpeedDialKeys[NameKeyIndex];
				if (Config.SpeedDialMonitored(k.Index))
				{
					// key is one of the monitored speed dials. 
					
					// If we're not paused and not
					// all monitored speed dial keys are Inactive (state<>active) send the
					// pause command to the controlled application
					// Otherwise if we've send the pause command before and all monitored
					// speed dial keys are inactive send the unpause command to the 
					// controlled Application
					if (   !bPaused
						&& !AllSpeedDialsInactive())
					{
						Config[Config.ControlledApplication].Pause();
						statusBar1.Panels[0].Text = "Paused";
						bPaused = true;
					}
					else if (   bPaused
						&& AllSpeedDialsInactive())
					{
						Config[Config.ControlledApplication].UnPause();
						statusBar1.Panels[0].Text = "";
						bPaused = false;
					}
				}
			}
		}
			
		/// <summary>
		/// Called when a line's state has changed
		/// </summary>
		/// <param name="LineIdx">index of line which has changed its state</param>
		public void OnLineStateChanged(int LineIdx)
		{
			CLMGRLib.CLMgrLineStates state = (CLMGRLib.CLMgrLineStates)Lines[LineIdx].DispState;
			Console.WriteLine("Line " + LineIdx + ", State: " + state);

			// If we've not send the pause command before and if the changed line's new state 
			// is something other than "Inactive" and not all lines are inactive, send the 
			// pause command to the controlled application
			// Otherwise if we've send the pause command before and if the changed line's
			// new state is "Inactive" and if all other lines are inactive, send the unpause
			// command to the controlled application
			if (   !bPaused 
				&& state!=CLMGRLib.CLMgrLineStates.LSInactive
				&& !AllLinesInactive())
			{
				Config[Config.ControlledApplication].Pause();
				statusBar1.Panels[0].Text = "Paused";
				bPaused = true;
			}
			else if (   bPaused 
				&& state==CLMGRLib.CLMgrLineStates.LSInactive
				&& AllLinesInactive())
			{
				Config[Config.ControlledApplication].UnPause();
				statusBar1.Panels[0].Text = "";
				bPaused = false;
			}		
		}
		
		/// <summary>
		/// Checks if all lines of the PBX client are inactive
		/// </summary>
		/// <returns>true if all lines in state inactive, false otherwise</returns>
		private bool AllLinesInactive()
		{
			// check if all other lines are inactive
			bool bAllInactive = true;
			for(int i=0;i<clmgr.DispNumberOfLines;i++)
			{
				if (Lines[i].DispState != (int)CLMGRLib.CLMgrLineStates.LSInactive)
				{
					bAllInactive=false;
					break;
				}
			}
			return bAllInactive;
		}

		/// <summary>
		/// Checks if all monitored speed dial keys have a state other than "Active"
		/// </summary>
		/// <returns>true, if all monitored speed dials have a state other than "Active".
		/// False otherise.</returns>
		private bool AllSpeedDialsInactive()
		{
			bool bAllInactive = true;
			foreach(int i in Config.SpeedDialKeys)
			{
				if (clmgr.DispSpeedDialState(i)==3)
				{
					bAllInactive = false;
					break;
				}
			}
			return bAllInactive;
		}

		/// <summary>
		/// Tray icon clicked, restore main window
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void notifyIcon1_Click(object sender, System.EventArgs e)
		{
			this.WindowState = System.Windows.Forms.FormWindowState.Normal;
			this.Activate();
		}

		/// <summary>
		/// main window resized. When window is minimized remove from  taskbar
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void MainForm_Resize(object sender, System.EventArgs e)
		{
			if (this.WindowState==FormWindowState.Minimized)
				this.ShowInTaskbar = false;
			else
				this.ShowInTaskbar = true;
		}

		/// <summary>
		/// Options button clicked. Show the controlled applications option dialog, if any
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void buttonOptions_Click(object sender, System.EventArgs e)
		{
			Config[Config.ControlledApplication].ShowOptionsDlg();
		}

		/// <summary>
		/// Load main form. Loads configuration data and logs in to PBX server.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void MainForm_Load(object sender, System.EventArgs e)
		{
			string appdatafolder = System.Environment.GetFolderPath(System.Environment.SpecialFolder.ApplicationData);
			appdatafolder = appdatafolder + "\\IpPbx";
			if (!System.IO.Directory.Exists(appdatafolder))
			{
				Directory.CreateDirectory(appdatafolder);
			}

			ConfigFilename = appdatafolder + "\\IpPbxMPCConfig.xml";

			LoadConfig();

			comboBoxApp.SelectedIndex = (int)Config.ControlledApplication;

			bLoggedIn = ConnectAndLogin();
			if (!bLoggedIn)
			{
				groupBox1.Enabled = false;
			}
			else
				groupBox1.Enabled = true;

		}

		/// <summary>
		/// Connect to PBX server and login. Additionally this method retrieves the 
		/// speed dial key definitions and line objects and initializes the corresponding
		/// form members
		/// </summary>
		/// <returns>true if successfully logged in. False otherwise</returns>
		bool ConnectAndLogin()
		{
			if (clmgr!=null)
			{
				//clmgr.DispReleaseUser(UserID);
				clmgr = null;
			}

			clmgr = new ClientLineMgrClass();
			clmgr.DispOnLineMgrNotification += 
				new CLMGRLib.IClientLineMgrEventsDisp_DispOnLineMgrNotificationEventHandler(this.OnLineMgrNotification);


			// init and register user
            /*
			if (Config.AutoServerSelection)
				clmgr.DispInitEx("","");
			else
				clmgr.DispInitEx(Config.ServerName,Config.FallbackServerName);
			if (Config.UseUserName)
				UserID = clmgr.DispRegisterUser(Config.UserName);
			else
				UserID = clmgr.DispRegisterUser("");
			
			if (UserID==0)
			{
				MessageBox.Show(this,"Login failed.",this.Text,MessageBoxButtons.OK,MessageBoxIcon.Error);
				UpdateStatusBar("Logged Off","","");
				return false;
			}
            */

            UpdateStatusBar("Initialized",clmgr.DispGetCurrentServer,clmgr.DispGetCurrentUser);
					
			// get lines
			Lines = new CLMGRLib.ClientLine[clmgr.DispNumberOfLines];
			for (int i=0;i<clmgr.DispNumberOfLines;i++)
			{
				Lines[i] = (CLMGRLib.ClientLine)clmgr.DispGetLine(i);
			}

			// get speed dial names
			int NumKeys = clmgr.DispNumberOfSpeedDials;
			SpeedDialKeys = new SpeedDialKey[NumKeys];
			checkedListBoxSpeedDials.Items.Clear();
			int j = 0;
			for (int i=0;i<NumKeys;i++)
			{
				if (clmgr.DispSpeedDialName(i)!="")
				{
					SpeedDialKey NewKey = new SpeedDialKey();
					NewKey.Index = i;
					NewKey.Name = clmgr.DispSpeedDialName(i);
					NewKey.Number = clmgr.DispSpeedDialNumber(i);
					SpeedDialKeys[j] = NewKey;

					// fill checkedlistbox
					if (Config.SpeedDialMonitored(i))
						checkedListBoxSpeedDials.Items.Add(NewKey,CheckState.Checked);
					else
						checkedListBoxSpeedDials.Items.Add(NewKey,CheckState.Unchecked);
					j++;
				}
			}

			if (Config.EventToReactOn==IpPbxMPCConfig.IpPbxEvents.SpeedDialKeyState)
			{
				radioButtonSpeedDialKey.Checked = true;
				checkedListBoxSpeedDials.Enabled = true;
			}
			else 
			{
				radioButtonLineState.Checked = true;
				checkedListBoxSpeedDials.Enabled = false;
			}

			return true;
		}

		/// <summary>
		/// Update status bar panels.
		/// </summary>
		/// <param name="state">State to show on status bar.</param>
		/// <param name="server">server name to show on status bar.</param>
		/// <param name="user">PBX user name to show on status bar.</param>
		void UpdateStatusBar(string state,string server, string user)
		{
			statusBar1.Panels[1].Text = state;
			statusBar1.Panels[2].Text = server;
			statusBar1.Panels[3].Text = user;
		}

		/// <summary>
		/// The "use Speed dial key state" radio button has changed it's checked state.
		/// Updates the configuration data
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void radioButtonSpeedDialKey_CheckedChanged(object sender, System.EventArgs e)
		{
			if (radioButtonSpeedDialKey.Checked)
			{
				checkedListBoxSpeedDials.Enabled = true;
				Config.EventToReactOn = IpPbxMPCConfig.IpPbxEvents.SpeedDialKeyState;
			}
			else
			{
				checkedListBoxSpeedDials.Enabled = false;
				Config.EventToReactOn = IpPbxMPCConfig.IpPbxEvents.NonActiveLineKey;
			}
		}

		/// <summary>
		/// Main form is closing. Update configuration data.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void MainForm_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			// get checked items from listbox
			if (Config != null)
			{
				Config.SpeedDialKeys.Clear();
				foreach(SpeedDialKey spdk in checkedListBoxSpeedDials.CheckedItems)
				{
					Config.SpeedDialKeys.Add(spdk.Index);
				}
			}
		}

		/// <summary>
		/// The checkbox of an item in the speed dial key list has changed its checked state.
		/// Update configuration data
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void checkedListBoxSpeedDials_ItemCheck(object sender, System.Windows.Forms.ItemCheckEventArgs e)
		{
			SpeedDialKey k = SpeedDialKeys[e.Index];
			if (k==null)
				return;

			if (   !Config.SpeedDialMonitored(k.Index)
				&& e.NewValue==CheckState.Checked)
			{
				Config.SpeedDialKeys.Add(k.Index);
			}
			else if (Config.SpeedDialMonitored(k.Index)
				&& e.NewValue==CheckState.Unchecked)
			{
				int i = Config.SpeedDialKeys.BinarySearch(k.Index);
				if (i!=-1)
					Config.SpeedDialKeys.RemoveAt(i);
			}

		}

		/// <summary>
		/// Handle "Advanced" button click. Shows AdvancedDlg and update configuration.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void buttonAdvanced_Click(object sender, System.EventArgs e)
		{
			AdvancedDlg dlg = new AdvancedDlg();
			dlg.ServerName = Config.ServerName;
			dlg.FallbackServer = Config.FallbackServerName;
			dlg.AutoServerSelection = Config.AutoServerSelection;
			dlg.UseUserName = Config.UseUserName;
			dlg.UserName = Config.UserName;
			DialogResult r = dlg.ShowDialog(this);
			
			if (r==DialogResult.OK)
			{
				// check if something has changed
				bool bChanged =    (Config.ServerName!=dlg.ServerName)
							    || (Config.FallbackServerName != dlg.FallbackServer)
				                || (Config.AutoServerSelection != dlg.AutoServerSelection)
							    || (Config.UseUserName != dlg.UseUserName)
							    || (Config.UserName != dlg.UserName);

				Config.ServerName = dlg.ServerName;
				Config.FallbackServerName = dlg.FallbackServer;
				Config.AutoServerSelection = dlg.AutoServerSelection;
				Config.UseUserName = dlg.UseUserName;
				Config.UserName = dlg.UserName;

				if (bChanged)
				{
					// something has changed, logoff and logon again
					if (ConnectAndLogin())
						groupBox1.Enabled = true;
					else
						groupBox1.Enabled = false;
				}
			}
		}

		/// <summary>
		/// PictureBox clicked. Show AboutDlg
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void pictureBox1_Click(object sender, System.EventArgs e)
		{
			AboutDlg dlg = new AboutDlg();
			dlg.Show();
		}
	}
}
