using System;

namespace IpPbxMPC
{
	/// <summary>
	/// Abstract base class for application specific configuration.
	/// For each controlled application type a configuration class
	/// is derived from this one.
	/// </summary>
	/// <remarks>We cannot use an interface class here, because the 
	/// XML serializer cannot handle that.</remarks>
	public abstract class IpPbxApp
	{
		/// <summary>
		/// Function to call to send application a "Pause" command
		/// </summary>
		public abstract void Pause();

		/// <summary>
		/// Function to call to send application an "UnPause" command
		/// </summary>
		public abstract void UnPause();

		/// <summary>
		/// Function to call to display an dialog to configure the application's
		/// configuration data
		/// </summary>
		public abstract bool ShowOptionsDlg();

		/// <summary>
		/// Determines if application has a configuration dialog
		/// </summary>
		/// <returns>true, if application has a configuration dialog. False otherwise</returns>
		public abstract bool HasOptions();
	}
}
