using System;
using System.Xml;
using System.Xml.Serialization;

namespace IpPbxMPC
{
	/// <summary>
	/// Configuration class for using Browseamp Winamp webbrowser plugin to control winamp.
	/// </summary>
	/// <remarks>May be used for any other media player which can be controlled by sending a
	/// http get request</remarks>
	[XmlRootAttribute("IpPbxBrowseAmp", Namespace="http://www.lanphone.de/IpPbxMPCConfig")]
	public class IpPbxBrowseAmp : IpPbxMPC.IpPbxApp
	{
		public IpPbxBrowseAmp()
		{
		}

		/// <summary>
		/// URL to request to pause playing
		/// </summary>
		public string PauseURL;

		/// <summary>
		/// URL to request to resume playing
		/// </summary>
		public string UnPauseURL;

		/// <summary>
		/// pause playing. Sends http get request with PauseURL 
		/// </summary>
		public override void Pause()
		{
			try
			{
				System.Net.WebRequest myRequest = System.Net.WebRequest.Create(this.PauseURL);
				System.Net.WebResponse response = myRequest.GetResponse();
				response.Close();
			}
			catch (System.Exception e)
			{
				Console.WriteLine(e);
				if (e.InnerException!=null)
					Console.WriteLine(e.InnerException);
			}
		}

		/// <summary>
		/// resume playing. Sends http get request with UnPauseURL 
		/// </summary>
		public override void UnPause()
		{
			try
			{
				System.Net.WebRequest myRequest = System.Net.WebRequest.Create(this.UnPauseURL);
				System.Net.WebResponse response = myRequest.GetResponse();
				response.Close();
			}
			catch (System.Exception e)
			{
				Console.WriteLine(e);
				if (e.InnerException!=null)
				Console.WriteLine(e.InnerException);
			}
		}

		/// <summary>
		/// Display options dialog to allow configuration of PauseURL and UnPauseURL
		/// </summary>
		/// <returns>true, if dialog was closed by pressing the ok button or false otherwise.</returns>
		public override bool ShowOptionsDlg()
		{
			BrowseAmpOptions dlg = new BrowseAmpOptions();
			dlg.PauseURL = PauseURL;
			dlg.UnPauseURL = UnPauseURL;
			System.Windows.Forms.DialogResult r = dlg.ShowDialog();
			if (r==System.Windows.Forms.DialogResult.OK)
			{
				PauseURL = dlg.PauseURL;
				UnPauseURL = dlg.UnPauseURL;
				return true;
			}
			return false;
		}

		/// <summary>
		/// Determine if an option dialog is available
		/// </summary>
		/// <returns>true</returns>
		public override bool HasOptions()
		{
			return true;
		}
	}
}	



