using System;
using System.Xml;
using System.Xml.Serialization;
using System.Diagnostics;


namespace IpPbxMPC
{
	/// <summary>
	/// Configuration class for "generic" application, i.e. any application that 
	/// can handle multimedia keys. Will send WM_APPCOMMAND window messages
	/// </summary>
	[XmlRootAttribute("IpPbxGeneric", Namespace="http://www.lanphone.de/IpPbxMPCConfig", 
		 IsNullable = false)]
	public class IpPbxGeneric : IpPbxMPC.IpPbxApp
	{
		[XmlIgnore]
		private System.Diagnostics.Process ControlledProcess;

		/// <summary>
		/// Name of process which will receive the WM_APPCOMMAND messages
		/// </summary>
		public string processname;

		public IpPbxGeneric()
		{
			System.Diagnostics.Process[] procs =
				System.Diagnostics.Process.GetProcessesByName(processname);
			if (procs.Length>0)
				ControlledProcess = procs[0];
		}

		/// <summary>
		/// Pause playing by sending a APPCOMMAND_MEDIA_PAUSE
		/// </summary>
		public override void Pause()
		{
			if (ControlledProcess!=null)
			{
				Win32.PostMessage((uint)ControlledProcess.MainWindowHandle,
					0x0319/*WM_APPCOMMAND*/,
					(uint)ControlledProcess.MainWindowHandle,0x002f0000 /*APPCOMMAND_MEDIA_PAUSE*/);
			}
		}

		/// <summary>
		/// Resume playing by sending a APPCOMMAND_MEDIA_PLAY
		/// </summary>
		public override void UnPause()
		{
			if (ControlledProcess!=null)
			{
				Win32.PostMessage((uint)ControlledProcess.MainWindowHandle,
					0x0319/*WM_APPCOMMAND*/,
					(uint)ControlledProcess.MainWindowHandle,0x002e0000 /*APPCOMMAND_MEDIA_PLAY*/);
			}
		}	

		/// <summary>
		/// Display options dialog to configure the process to control
		/// </summary>
		/// <returns>true, if dialog was closed by pressing the ok button or false otherwise</returns>
		public override bool ShowOptionsDlg()
		{
			System.Diagnostics.Process[] procs =
				System.Diagnostics.Process.GetProcesses();

			IpPbxGenericOptions dlg = new IpPbxGenericOptions();
			for(int i=0;i<procs.Length;i++)
			{
				dlg[i] = procs[i].ProcessName;
			}
			dlg.SelectedProcess = processname;

			System.Windows.Forms.DialogResult r =
				dlg.ShowDialog();
			if (r==System.Windows.Forms.DialogResult.OK)
			{
				processname = dlg.SelectedProcess;
				System.Diagnostics.Process[] procs2 =
					System.Diagnostics.Process.GetProcessesByName(processname);
				if (procs.Length>0)
					ControlledProcess = procs2[0];
				return true;
			}
			return false;
		}

		/// <summary>
		/// Determines if option dialog available
		/// </summary>
		/// <returns>true</returns>
		public override bool HasOptions()
		{
			return true;
		}
	}
}
