using System;
using System.Xml;
using System.Xml.Serialization;

namespace IpPbxMPC
{
	/// <summary>
	/// Configuration class to control winamp.
	/// </summary>
	/// <remarks>Uses winamp specific WM_USER message to control winamp</remarks>
	[XmlRootAttribute("IpPbxWinamp", Namespace="http://www.lanphone.de/IpPbxMPCConfig", 
		 IsNullable = false)]
	public class IpPbxWinamp : IpPbxMPC.IpPbxApp
	{
		[XmlIgnore]
		private uint WinampWindow;

		[XmlIgnore]
		private bool bPaused;

		public IpPbxWinamp()
		{
			WinampWindow = Win32.FindWindow("Winamp v1.x",null);
			bPaused = false;
		}

		/// <summary>
		/// Pause winamp by sending winamp specific WM_USER message.
		/// </summary>
		/// <remarks>Will do nothing if winamp is not playing.</remarks>
		public override void Pause()
		{
			if (WinampWindow==0)
				WinampWindow = Win32.FindWindow("Winamp v1.x",null);

			if (WinampWindow!=0 && !bPaused) 
			{
				// check if winamp is currently playing
				uint r = Win32.SendMessage(WinampWindow,0x400/*WM_USER*/,0,104);
				if (r==1)
				{
					// pause
					Win32.PostMessage(WinampWindow, 
						0x111 /*WM_COMMAND*/,
						40046,0);
					bPaused = true;
				}
			}
		}

		/// <summary>
		/// Resume winamp by sending winamp specific WM_USER message.
		/// </summary>
		/// <remarks>Will do nothing if winamp was not paused by this class.</remarks>
		public override void UnPause()
		{
			if (WinampWindow==0)
				WinampWindow = Win32.FindWindow("Winamp v1.x",null);

			if (WinampWindow!=0 && bPaused)
			{
				// check if winamp is currently paused
				uint r = Win32.SendMessage(WinampWindow,0x400/*WM_USER*/,0,104);
				if (r==3)
				{
					// unpause
					Win32.PostMessage(WinampWindow, 
						0x111 /*WM_COMMAND*/,
						40046,0);
					bPaused = false;
				}
			}
		}	

		/// <summary>
		/// Show options dialog. None available right now.
		/// </summary>
		/// <returns>false</returns>
		public override bool ShowOptionsDlg()
		{
			return false;
		}

		/// <summary>
		/// Determine if options dialog is available
		/// </summary>
		/// <returns>false</returns>
		public override bool HasOptions()
		{
			return false;
		}
	}
}
