﻿//-----------------------------------------------------------------------
// <copyright file="IClientAddInLoader.cs" company="Swyx Solutions AG">
//     (c) Swyx Solutions AG
// </copyright>
//-----------------------------------------------------------------------
using System;
using System.Diagnostics.CodeAnalysis;
using System.Runtime.InteropServices;
using System.Text;

namespace CSharpAddIn
{
    /// <summary>
    /// Line Manager PlugIn interface
    /// called by Client Line Manager when loading / unloading the plugin
    /// This interface has to be implemented by all Line Manager PlugIns
    /// </summary>
    [ComVisible(true)]
    [InterfaceType(ComInterfaceType.InterfaceIsIUnknown),
     Guid("f8e55372-4c00-11d3-80bc-00105a653379")]
    [AutomationProxy(false)]
    public interface IClientAddInLoader
    {
        /// <summary>
        /// This method will be called after loading the PlugIn
        /// </summary>
        /// <param name="reserved">Reserved, must be zero</param>
        /// <returns>0 if successful, error code otherwise</returns>
        /// <remarks>If the PlugIn connects to the client line manager for receiving 
        /// events, it should establish the connection in response to "Initialize"
        /// Calling PubInit, PubRegisterUser and PubReleaseUser is not required for PlugIns.
        /// Registration should be done only by full clients, clients that intend to 
        /// provide a complete phone user interface like SwyxIt! itself.</remarks>
        [PreserveSig()]
        [return: MarshalAs(UnmanagedType.Error)]
        int Initialize([MarshalAs(UnmanagedType.U4), In()]int reserved);

        /// <summary>
        /// This method returns the visible name of the PlugIn
        /// </summary>
        /// <param name="name">Plugin name</param>
        /// <returns>0 if successful, error code otherwise</returns>
        [SuppressMessage("Microsoft.Design", "CA1021", Justification = "We have to implement a legacy COM interface requiring this signature")]
        [PreserveSig()]
        [return: MarshalAs(UnmanagedType.Error)]
        int GetName([MarshalAs(UnmanagedType.BStr), Out()]out string name);

        /// <summary>
        /// This method returns the visible version information of the PlugIn
        /// </summary>
        /// <param name="version">Plugin version</param>
        /// <returns>0 if successful, error code otherwise</returns>
        [SuppressMessage("Microsoft.Design", "CA1021", Justification = "We have to implement a legacy COM interface requiring this signature")]
        [PreserveSig()]
        [return: MarshalAs(UnmanagedType.Error)]
        int GetVersion([MarshalAs(UnmanagedType.BStr), Out()]out string version);

        /// <summary>
        /// This method will be called before releasing the PlugIn
        /// </summary>
        /// <param name="reserved">Reserved, must be zero</param>
        /// <returns>0 if successful, error code otherwise</returns>
        /// <remarks>If the PlugIn connects to the client line manager for receiving 
        /// events, it should release the connection in response to "UnInitialize" 
        /// Calling PubInit, PubRegisterUser and PubReleaseUser is not required for 
        /// PlugIns.</remarks>
        [SuppressMessage("Microsoft.Naming", "CA1702", Justification = "We have to implement a legacy COM interface requiring this signature")]
        [SuppressMessage("Microsoft.Naming", "CA1709", Justification = "We have to implement a legacy COM interface requiring this signature")]
        [PreserveSig()]
        [return: MarshalAs(UnmanagedType.Error)]
        int UnInitialize([MarshalAs(UnmanagedType.U4), In()]int reserved);
    }
}
