﻿//-----------------------------------------------------------------------
// <copyright file="IClientResolverAddIn.cs" company="Swyx Solutions AG">
//     (c) Swyx Solutions AG
// </copyright>
//-----------------------------------------------------------------------
using System;
using System.Diagnostics.CodeAnalysis;
using System.Runtime.InteropServices;
using System.Text;

namespace CSharpAddIn
{
    /// <summary>
    /// Preferred number style enumeration
    /// </summary>
    public enum PreferredNumberStyle
    {
        /// <summary>
        /// Defines a full number, e.g. 0004923147770 or 0023147770
        /// </summary>
        Full = 0,

        /// <summary>
        /// Defines a  plain number, e.g. 4923147770 or 23147770
        /// </summary>
        Plain = 1,

        /// <summary>
        /// Defines a canonical number, e.g. +49 (231) 47770 or +49 (231) 4777-0
        /// </summary>
        Canonical = 2
    }

    /// <summary>
    /// Line Manager PlugIn interface; used by Client Line Manager for resolving
    /// a phone number to a name
    /// This interface has to be implemented by a Line Manager PlugIn if it allows
    /// resolving phone numbers.
    /// </summary>
    [ComVisible(true)]
    [InterfaceType(ComInterfaceType.InterfaceIsIUnknown),
     Guid("f8e55371-4c00-11d3-80bc-00105a653379"),
     AutomationProxy(false)]
    public interface IClientResolverAddIn
    {
        /// <summary>
        /// This method finds a phone number to a name
        /// </summary>
        /// <param name="number">Number to search for. Format will be preferred number style as 
        /// returned by GetPreferredNumberStyle</param>
        /// <param name="name">Name to resolve</param>
        /// <returns>If the function succeeds, Name contains the name for the number and the
        /// return value is S_OK (0). If there's no name for the number, the name is empty and 
        /// the function returns S_FALSE (1)
        /// IF the number is empty the function returns E_INVALIDARG (0x80070057)</returns>
        [SuppressMessage("Microsoft.Design", "CA1021", Justification = "We have to implement a legacy COM interface requiring this signature")]
        [PreserveSig()]
        [return: MarshalAs(UnmanagedType.Error)]
        int ResolveNumber(
            [MarshalAs(UnmanagedType.BStr), In()]string number, 
            [MarshalAs(UnmanagedType.BStr), Out()]out string name);

        /// <summary>
        /// Returns the preferred number style to be resolved
        /// </summary>
        /// <param name="preferredNumberStyle">Preferred number style</param>
        /// <returns>If the function succeeds, it returns S_OK (0)</returns>
        [return: MarshalAs(UnmanagedType.Error)]
        [PreserveSig()]
        [SuppressMessage("Microsoft.Design", "CA1021", Justification = "We have to implement a legacy COM interface requiring this signature")]
        int GetPreferredNumberStyle([MarshalAs(UnmanagedType.U4)]out int preferredNumberStyle);
    }
}
