﻿//-----------------------------------------------------------------------
// <copyright file="IFulltextSearchAddIn.cs" company="Swyx Solutions AG">
//     (c) Swyx Solutions AG
// </copyright>
//-----------------------------------------------------------------------
using System;
using System.Diagnostics.CodeAnalysis;
using System.Runtime.InteropServices;

namespace CSharpAddIn
{
    /// <summary>
    /// Line Manager PlugIn interface; used by Client Line Manager for resolving
    /// a string fragment to several search results (i.e. fulltext search in contacts; name or numbers)
    /// This interface has to be implemented by a Line Manager PlugIn if it allows a "text search".
    /// </summary>
    [SuppressMessage("Microsoft.Naming", "CA1704", Justification = "We have to implement a legacy COM interface requiring this signature")]
    [ComVisible(true)]
    [InterfaceType(ComInterfaceType.InterfaceIsIUnknown),
     Guid("f8e55548-4c00-11d3-80bc-00105a653379"),
     AutomationProxy(false)]
    public interface IFulltextSearchAddIn
    {
        /// <summary>
        /// This method will start a full text search in name or numbers in contacts
        /// </summary>
        /// <param name="searchText">Text fragment to be searched</param>
        /// <param name="enableSearchInNumbers">if set to TRUE, the search string will 
        /// be searched in numbers, too, BUT ONLY if it contains of solely digits. 
        /// If there is just one non-digit character in the search string, the search is 
        /// carried out only in the contact's names, as it will be if the variable 
        /// is set to FALSE</param>
        /// <param name="searchResultCollection">Search result collection to fill with results</param>
        /// <returns>If the function succeeds, the result is zero (S_OK)</returns>
        [SuppressMessage("Microsoft.Design", "CA1021", Justification = "We have to implement a legacy COM interface requiring this signature")]
        [SuppressMessage("Microsoft.Naming", "CA1704", Justification = "We have to implement a legacy COM interface requiring this signature")]
        [PreserveSig()]
        [return: MarshalAs(UnmanagedType.Error)]
        int FulltextSearchInContacts(
            [MarshalAs(UnmanagedType.BStr), In()] string searchText,
            [MarshalAs(UnmanagedType.Bool), In()] bool enableSearchInNumbers,
            [MarshalAs(UnmanagedType.IDispatch), In(), Out()] object searchResultCollection);
    }
}
