/*---------------------------------------------------------------------------

   Swyx Solutions AG
   Joseph-von-Fraunhofer-Str. 13a
   44227 Dortmund

  ---------------------------------------------------------------------------

  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
  PURPOSE.

  Copyright (C) 2007 Swyx Solutions AG.

  All Rights Reserved.

----------------------------------------------------------------------------*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using CLMGRLib;

namespace Simple
{
    public partial class FrmMain : Form
    {
        //the ClientLineMgrClass is the core object for working with the client sdk
        private ClientLineMgrClass MyCLMgr = new ClientLineMgrClass();
        private ClientSdkEvents.ClientSdkEventSink MyEventSink = new ClientSdkEvents.ClientSdkEventSink();

        bool PlayingOutboundTestSignal = false;
        bool PlayingInboundTestSignal = false;
        int OutboundTestSignalLine = -1;
        int InboundTestSignalLine = -1;
        int TestCounter = 0;

        bool PickedUpIncomingCall = false;
        System.DateTime PickupTimeStamp;



        public FrmMain()
        {
            InitializeComponent();            
        }

        private void FrmMain_Load(object sender, EventArgs e)
        {
            //Needed to prevent Threading Exceptions (do thread handling like VS2003)
            Control.CheckForIllegalCrossThreadCalls = true;

            MyEventSink.Connect(MyCLMgr, this, new ClientSdkEvents.LineManagerMessageHandler(OnLineManagerMessage));

            MyCLMgr.SetLineCount(2);

            textBoxCallerId.Text = "";
            textBoxDialNumber.Text = "";
            textBoxCallerSendDtmf.Text = "";
            textBoxCallerReceivedDtmf.Text = "";
            textBoxCalledReceivedDtmf.Text = "";
            checkBoxCallerPlaySound.Checked = true;
            checkBoxCalledPlaySound.Checked = true;
        }

        private void FrmMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            MyEventSink.Disconnect();
        }

        private void OnLineManagerMessage(ClientSdkEvents.ClientSdkEventArgs e)
        {

            ClientLine NotifiedLine = null;

            //check what message we got
            switch (e.Msg)
            {
                case ClientSdkEvents.CLMgrMessage.CLMgrNumberOfLinesChangedMessage:
                    //do something about
                    break;

                // state of a line has been changed
                case ClientSdkEvents.CLMgrMessage.CLMgrLineStateChangedMessageEx:
                    {
                        int line = e.Param & 0xff;
                        int high = e.Param >> 8;

                        ClientSdkEvents.LineState NewLineState = (ClientSdkEvents.LineState)high;

                        NotifiedLine = (ClientLine)MyCLMgr.DispGetLine(line); ;

                        //Check which LineState we have (see enum for description)
                        switch (NewLineState)
                        {
                            case ClientSdkEvents.LineState.Inactive:
                                NotifiedLine.DispStopPlaySoundFile();
                                NotifiedLine.DispStopRecordSoundFile();
                                break;
                            case ClientSdkEvents.LineState.Alerting:
                                break;
                            case ClientSdkEvents.LineState.Busy:
                                break;
                            case ClientSdkEvents.LineState.ConferenceActive:
                                break;
                            case ClientSdkEvents.LineState.ConferenceOnHold:
                                break;
                            case ClientSdkEvents.LineState.Dialing:
                                break;
                            case ClientSdkEvents.LineState.Disabled:
                                break;
                            case ClientSdkEvents.LineState.HookOffExternal:
                                break;
                            case ClientSdkEvents.LineState.HookOffInternal:
                                break;
                            case ClientSdkEvents.LineState.Knocking:
                                break;
                            case ClientSdkEvents.LineState.OnHold:
                                break;
                            case ClientSdkEvents.LineState.Ringing:
                                {
                                    //if call from number 778, accept call
                                    textBoxCallerIdInbound.Text = NotifiedLine.DispPeerNumber;
                                }
                                break;
                            case ClientSdkEvents.LineState.Active:
                                {
                                    if (NotifiedLine.DispIsOutgoingCall!=0)
                                    {
                                        if (PickedUpIncomingCall)
                                        {
                                            PickedUpIncomingCall = false;
                                            System.TimeSpan delta = System.DateTime.Now - PickupTimeStamp;
                                            textBoxTimeDelta.Text = delta.Milliseconds.ToString();

                                            if (!PlayingOutboundTestSignal)
                                            {
                                                if (checkBoxCallerPlaySound.Checked)
                                                {
                                                    PlayingOutboundTestSignal = true;
                                                    OutboundTestSignalLine = line;

                                                    string sPlayFilename = Application.StartupPath + "\\CallingPartyTestsignal1000Hz10Seconds.wav";

                                                    NotifiedLine.DispPlaySoundFile(sPlayFilename, 0, 0);
                                                }
                                            }
                                        }
                                    }
                                }
                                break;

                            case ClientSdkEvents.LineState.Terminated:
                                {
                                    //if we have a Line terminated we may want to know the reason
                                    ClientSdkEvents.DisconnectReason m_DiscReason = (ClientSdkEvents.DisconnectReason)NotifiedLine.DispDiscReason;

                                    //special handling required? who knows
                                    switch (m_DiscReason)
                                    {
                                        case ClientSdkEvents.DisconnectReason.Busy:
                                            break;
                                        case ClientSdkEvents.DisconnectReason.CallDiverted:
                                            break;
                                        case ClientSdkEvents.DisconnectReason.CallRoutingFailed:
                                            break;
                                        case ClientSdkEvents.DisconnectReason.Cancelled:
                                            break;
                                        case ClientSdkEvents.DisconnectReason.DirectCallImpossible:
                                            break;
                                        case ClientSdkEvents.DisconnectReason.IncompatibleDestination:
                                            break;
                                        case ClientSdkEvents.DisconnectReason.JoinedConference:
                                            break;
                                        case ClientSdkEvents.DisconnectReason.NetworkCongestion:
                                            break;
                                        case ClientSdkEvents.DisconnectReason.NoAnswer:
                                            break;
                                        case ClientSdkEvents.DisconnectReason.NoChannelAvailable:
                                            break;
                                        case ClientSdkEvents.DisconnectReason.NumberChanged:
                                            break;
                                        case ClientSdkEvents.DisconnectReason.PermissionDenied:
                                            break;
                                        case ClientSdkEvents.DisconnectReason.Rejected:
                                            break;
                                        case ClientSdkEvents.DisconnectReason.TooLate:
                                            break;
                                        case ClientSdkEvents.DisconnectReason.Transferred:
                                            break;
                                        case ClientSdkEvents.DisconnectReason.Unreachable:
                                            break;
                                        case ClientSdkEvents.DisconnectReason.WrongNumber:
                                            break;
                                        case ClientSdkEvents.DisconnectReason.Normal:
                                            break;
                                    }
                                }
                                break;
                        }
                    }
                    break;

                case ClientSdkEvents.CLMgrMessage.CLMgrLineDetailsChangedMessage:
                    {
                        //name, number, dtmf, ... might have changed
                        int index = e.Param;

                        //show how to handle DTMF
                        NotifiedLine = (ClientLine)MyCLMgr.DispGetLine(index);

                        string dtmf=NotifiedLine.DispReceivedDtmfDigits;

                        if (NotifiedLine.DispIsOutgoingCall!=0)
                        {
                            textBoxCallerReceivedDtmf.Text = dtmf;
                        }
                        else
                        {
                            if (dtmf.Length > textBoxCalledReceivedDtmf.Text.Length)
                            {
                                string newDtmf = dtmf.Substring(textBoxCalledReceivedDtmf.Text.Length);
                                if (checkBoxCalledMirrorDtmf.Checked)
                                {
                                    int iInband=0;
                                    if (checkBoxInbandDtmf.Checked)
                                    {
                                        iInband = 1;
                                    }
                                    NotifiedLine.DispSendDtmf(newDtmf, iInband);
                                }
                            }
                            textBoxCalledReceivedDtmf.Text = dtmf;
                        }
                    }
                    break;

                case ClientSdkEvents.CLMgrMessage.CLMgrWaveFilePlayed:
                    {
                        int index = e.Param;

                        //this tells us that 
                        if (index != -1)
                        {
                            NotifiedLine = (ClientLine)MyCLMgr.DispGetLine(index);

                            if (checkBoxCallerPlaySound.Checked)
                            {
                                if (PlayingOutboundTestSignal && (index == OutboundTestSignalLine))
                                {
                                    PlayingOutboundTestSignal = false;
                                    OutboundTestSignalLine = -1;

                                    NotifiedLine.DispStopPlaySoundFile();
                                    NotifiedLine.DispHookOn();
                                }
                            }
                            else if (checkBoxCalledPlaySound.Checked)
                            {
                                if(PlayingInboundTestSignal && (index==InboundTestSignalLine))
                                {
                                    PlayingInboundTestSignal = false;
                                    InboundTestSignalLine = -1;

                                    NotifiedLine.DispStopPlaySoundFile();
                                    NotifiedLine.DispHookOn();
                                }
                            }
                        }
                    }
                    break;

                case ClientSdkEvents.CLMgrMessage.CLMgrSIPRegistrationStateChanged:
                    //Get Line
                    {
                        int index = e.Param & 0xff;
                        int registrationstate = e.Param >> 8;
                    }
                    break;
            }
        }

        //Click on Button 1 -> do outbound call
        private void cmd_Line1_Click(object sender, EventArgs e)
        {
            ClientLine WorkingLine = null;
            ClientSdkEvents.LineState ls;
            bool bDone=false;

            int NumberOfLines=MyCLMgr.DispNumberOfLines;

            for(int i=0; i<NumberOfLines; i++)
            {
                WorkingLine=(ClientLine)MyCLMgr.DispGetLine(i);

                //we need the LineState of Line 1
                ls = (ClientSdkEvents.LineState)WorkingLine.DispState;

                //handle the Line based on its current LineState
                switch (ls)
                {
                    //if inactive and no number / name in the textfield just hook off
                    case ClientSdkEvents.LineState.Inactive:
                        {
                            //create a media streaming link and link this line
                            string MsGuid = MyCLMgr.DispCreateMediastreamingLink();

                            WorkingLine.DispSetMediastreamingLink(MsGuid);

                            //we are in this test case no longer interested; usually we would remember the GUID for later use
                            //nevertheless this will not destroy the linked media streaming since there are still lines using that link
                            MyCLMgr.DispDeleteMediastreamingLink(MsGuid);

                            TestCounter++;

                            string sRecordFilename = Application.StartupPath
                                                    + "\\"
                                                    + TestCounter.ToString()
                                                    + " A "
                                                    + textBoxCallerId.Text 
                                                    + " - " 
                                                    + textBoxDialNumber.Text 
                                                    + " - " 
                                                    + System.DateTime.Now.ToLongTimeString().Replace(":", "") 
                                                    + ".wav";
                            sRecordFilename = sRecordFilename;

                            WorkingLine.DispRecordSoundFile(sRecordFilename, 0);

                            PickedUpIncomingCall = false;

                            WorkingLine.DispHookOffEx(textBoxCallerId.Text, 0, textBoxCallerId.Text);
                            WorkingLine.DispDial(textBoxDialNumber.Text);
                            bDone=true;
                            textBoxCallerReceivedDtmf.Text = "";
                        }
                        break;

                    // if we are off hook or dialing or alerting then hook on again
                    case ClientSdkEvents.LineState.HookOffExternal:
                    case ClientSdkEvents.LineState.HookOffInternal:
                    case ClientSdkEvents.LineState.Dialing:
                    case ClientSdkEvents.LineState.Alerting:
                        break;
                    //if line is active set call on hold
                    case ClientSdkEvents.LineState.Active:
                        break;

                    //if line is on hold make the call active again
                    case ClientSdkEvents.LineState.OnHold:
                        break;
                    case ClientSdkEvents.LineState.Ringing:
                        break;
                }

                if(bDone)
                {
                    break;
                }
            }
        }

        //"accept call"
        private void cmd_Line2_Click(object sender, EventArgs e)
        {
            ClientLine WorkingLine = null;
            ClientSdkEvents.LineState ls;
            bool bDone = false;

            int NumberOfLines = MyCLMgr.DispNumberOfLines;

            for (int i = 0; i < NumberOfLines; i++)
            {
                WorkingLine = (ClientLine)MyCLMgr.DispGetLine(i);

                //we need the LineState of Line 1
                ls = (ClientSdkEvents.LineState)WorkingLine.DispState;

                //handle the Line based on its current LineState
                switch (ls)
                {
                    //if inactive and no number / name in the textfield just hook off
                    case ClientSdkEvents.LineState.Inactive:
                        break;

                    // if we are off hook or dialing or alerting then hook on again
                    case ClientSdkEvents.LineState.HookOffExternal:
                    case ClientSdkEvents.LineState.HookOffInternal:
                    case ClientSdkEvents.LineState.Dialing:
                    case ClientSdkEvents.LineState.Alerting:
                        break;
                    //if line is active set call on hold
                    case ClientSdkEvents.LineState.Active:
                        break;

                    //if line is on hold make the call active again
                    case ClientSdkEvents.LineState.OnHold:
                        break;
                    case ClientSdkEvents.LineState.Ringing:
                        {
                            //create a media streaming link and link this line
                            string MsGuid = MyCLMgr.DispCreateMediastreamingLink();

                            WorkingLine.DispSetMediastreamingLink(MsGuid);

                            //we are in this test case no longer interested; usually we would remember the GUID for later use
                            //nevertheless this will not destroy the linked media streaming since there are still lines using that link
                            MyCLMgr.DispDeleteMediastreamingLink(MsGuid);

                            textBoxCalledReceivedDtmf.Text = "";

                            WorkingLine.DispHookOff();

                            PickedUpIncomingCall = true;
                            PickupTimeStamp=System.DateTime.Now;


                            string sPlayFilename = Application.StartupPath + "\\CalledPartyTestsignal2000Hz10Seconds.wav";
                            string sRecordFilename = Application.StartupPath 
                                                    + "\\"
                                                    + TestCounter.ToString()
                                                    + " B "
                                                    + WorkingLine.DispCalledExtension 
                                                    + " - " 
                                                    + WorkingLine.DispPeerNumber 
                                                    + " - " 
                                                    + System.DateTime.Now.ToLongTimeString().Replace(":", "") 
                                                    + ".wav";

                            WorkingLine.DispRecordSoundFile(sRecordFilename, 0);

                            if (checkBoxCalledPlaySound.Checked)
                            {
                                WorkingLine.DispPlaySoundFile(sPlayFilename, 0, 0);
                                PlayingInboundTestSignal = true;
                                InboundTestSignalLine = i;
                            }

                            bDone = true;
                        }
                        break;
                }

                if (bDone)
                {
                    break;
                }
            }
        }

        //disconnect all calls
        private void cmd_Line3_Click(object sender, EventArgs e)
        {
            ClientLine WorkingLine = null;
            ClientSdkEvents.LineState ls;

            int NumberOfLines = MyCLMgr.DispNumberOfLines;

            for (int i = 0; i < NumberOfLines; i++)
            {
                WorkingLine = (ClientLine)MyCLMgr.DispGetLine(i);

                //we need the LineState of Line 1
                ls = (ClientSdkEvents.LineState)WorkingLine.DispState;

                //handle the Line based on its current LineState
                switch (ls)
                {
                    //if inactive and no number / name in the textfield just hook off
                    case ClientSdkEvents.LineState.Inactive:
                        WorkingLine.DispStopPlaySoundFile();
                        WorkingLine.DispStopRecordSoundFile();
                        WorkingLine.DispSetMediastreamingLink("");
                        break;

                    // if we are off hook or dialing or alerting then hook on again
                    case ClientSdkEvents.LineState.Ringing:
                    case ClientSdkEvents.LineState.HookOffExternal:
                    case ClientSdkEvents.LineState.HookOffInternal:
                    case ClientSdkEvents.LineState.Dialing:
                    case ClientSdkEvents.LineState.Alerting:
                    case ClientSdkEvents.LineState.Active:
                    case ClientSdkEvents.LineState.OnHold:
                    default:
                        {
                            WorkingLine.DispStopPlaySoundFile();
                            WorkingLine.DispStopRecordSoundFile();
                            WorkingLine.DispSetMediastreamingLink("");
                            WorkingLine.DispHookOn();
                        }
                        break;
                }
            }

            PlayingInboundTestSignal = false;
            InboundTestSignalLine = -1;
            PlayingOutboundTestSignal = false;
            OutboundTestSignalLine = -1;

        }

        
        private void cmd_LinkMedia_Click(object sender, EventArgs e)
        {
            //create a media streaming link and link all lines
            string MsGuid = MyCLMgr.DispCreateMediastreamingLink();

            ClientLine WorkingLine = null;

            int NumberOfLines=MyCLMgr.DispNumberOfLines;

            //link voice of all lines to that media streaming link
            for (int i = 0; i < NumberOfLines; i++)
            {
                WorkingLine = (ClientLine)MyCLMgr.DispGetLine(i);
                if (WorkingLine != null)
                {
                    WorkingLine.DispSetMediastreamingLink(MsGuid);
                }
            }

            //we are in this test case no longer interested; usually we would remember the GUID for later use
            //nevertheless this will not destroy the linked media streaming since there are still lines using that link
            MyCLMgr.DispDeleteMediastreamingLink(MsGuid); 
        }

        private void cmd_UnlinkMedia_Click(object sender, EventArgs e)
        {
            ClientLine WorkingLine = null;

            int NumberOfLines = MyCLMgr.DispNumberOfLines;

            //unlink all lines from the "media streaming link"
            for (int i = 0; i < NumberOfLines; i++)
            {
                WorkingLine = (ClientLine)MyCLMgr.DispGetLine(i);
                if (WorkingLine != null)
                {
                    WorkingLine.DispSetMediastreamingLink("");
                }
            }
        }

        private void cmd_Record_Click(object sender, EventArgs e)
        {
            //record on first line
            string sFilename = Application.StartupPath + "\\recordtest.wav";
            ClientLine s0;
            s0 = (ClientLine)MyCLMgr.DispGetLine(0);
            s0.DispRecordSoundFile(sFilename, 0);
        }

        private void cmd_StopRecord_Click(object sender, EventArgs e)
        {
            ClientLine s0;
            s0 = (ClientLine)MyCLMgr.DispGetLine(0);
            s0.DispStopRecordSoundFile();
        }

        private void cmd_Play_Click(object sender, EventArgs e)
        {
            string sFilename = Application.StartupPath + "\\SdkSample.wav";
            ClientLine s0;
            s0 = (ClientLine)MyCLMgr.DispGetLine(0);
            s0.DispPlaySoundFile(sFilename, 0, 0);
        }

        private void cmd_StopPlay_Click(object sender, EventArgs e)
        {
            ClientLine s0;
            s0 = (ClientLine)MyCLMgr.DispGetLine(0);
            s0.DispStopPlaySoundFile();
        }

        private void checkBoxInbandDtmf_CheckedChanged(object sender, EventArgs e)
        {
            int iDtmfDetectionMode=0;
            if (checkBoxInbandDtmf.Checked)
            {
                iDtmfDetectionMode = 4;
            }
            ClientLine WorkingLine = null;

            int NumberOfLines = MyCLMgr.DispNumberOfLines;

            //unlink all lines from the "media streaming link"
            for (int i = 0; i < NumberOfLines; i++)
            {
                WorkingLine = (ClientLine)MyCLMgr.DispGetLine(i);
                if (WorkingLine != null)
                {
                    WorkingLine.DispSetDtmfDetectionMode(iDtmfDetectionMode);
                }
            }
        }

        private void buttonCallerSendDtmf_Click(object sender, EventArgs e)
        {
            ClientLine WorkingLine = null;

            int NumberOfLines = MyCLMgr.DispNumberOfLines;

            //unlink all lines from the "media streaming link"
            for (int i = 0; i < NumberOfLines; i++)
            {
                WorkingLine = (ClientLine)MyCLMgr.DispGetLine(i);
                if (WorkingLine != null)
                {
                    if (WorkingLine.DispIsOutgoingCall!=0)
                    {
                        int iInband = 0;
                        if (checkBoxInbandDtmf.Checked)
                        {
                            iInband = 1;
                        }
                        WorkingLine.DispSendDtmf(textBoxCallerSendDtmf.Text, iInband);
                        break;
                    }
                }
            }

        }

    }
}




namespace ClientSdkEvents
{
    public class ClientSdkEventArgs : EventArgs
    {
        public CLMgrMessage Msg;
        public int Param;

        public ClientSdkEventArgs(CLMgrMessage msg, int param)
        {
            Msg = msg;
            Param = param;
        }
    }

    public delegate void LineManagerMessageHandler(ClientSdkEventArgs e);

    public class ClientSdkEventSink
    {
        private ClientLineMgrClass ConnectedLineManager;
        private IClientLineMgrEventsPub_PubOnLineMgrNotificationEventHandler EventHandler;
        private Form ConnectedForm;
        private LineManagerMessageHandler LineManagerMessageDelegateOfForm;

        public ClientSdkEventSink()
        {
            EventHandler = new IClientLineMgrEventsPub_PubOnLineMgrNotificationEventHandler(clmgr_EventSink);
        }

        public void Connect(ClientLineMgrClass lineManager, Form form, LineManagerMessageHandler lineManagerMessageDelegateOfForm)
        {
            ConnectedLineManager = lineManager;
            ConnectedForm = form;
            LineManagerMessageDelegateOfForm = lineManagerMessageDelegateOfForm;

            //add eventhandler for the PubOnlineMgrNotification Events
            ConnectedLineManager.PubOnLineMgrNotification += EventHandler;
        }

        public void Disconnect()
        {
            //remove eventhandler for the PubOnlineMgrNotification Events
            ConnectedLineManager.PubOnLineMgrNotification -= EventHandler;
            ConnectedLineManager = null;
            ConnectedForm = null;
            LineManagerMessageDelegateOfForm = null;
        }

        private void clmgr_EventSink(int msg, int param)
        {
            //this method receives the COM events from the client line manger
            if ((ConnectedForm != null) && (LineManagerMessageDelegateOfForm != null))
            {
                ConnectedForm.Invoke(LineManagerMessageDelegateOfForm, new ClientSdkEventArgs((CLMgrMessage)msg, param));
            }
        }
    }

    public enum CLMgrMessage
    {
        CLMgrLineStateChangedMessage = 0,      //state of at least one line has changed
        CLMgrLineSelectionChangedMessage = 1,  //line in focus has changed
        CLMgrLineDetailsChangedMessage = 2,    //details of at least one line have changed
        CLMgrCallDetailsMessage = 4,           //details of last call are available, post mortem for logging purpose
        CLMgrServerDownMessage = 5,            //server goes down, keep line manager, wait for ServerUp message
        CLMgrServerUpMessage = 6,              //server is up again, keep interfaces to line manger
        CLMgrWaveDeviceChanged = 7,            //speaker / micro has been switched on / off
        CLMgrGroupCallNotificationMessage = 8, //notification about group call
        CLMgrNumberOfLinesChangedMessage = 10, //the number of lines has changed
        CLMgrClientShutDownRequest = 11,       //Client Line Manager requests client to shutdown and release all interfaces
        CLMgrLineStateChangedMessageEx = 28,   //state of certain line has changed, lParam: LOWORD: line index of line that changed its state (starting with 0) HIWORD: new state of this line
        CLMgrSIPRegistrationStateChanged = 30, //registration state of SIP account has changed
        //lParam: LOBYTE: Account index
        //        HIBYTE: new state
        CLMgrWaveFilePlayed = 31,              //wave file playback finished
        //lParam: line index;
        //if -1, the message is related to a LineMgr function PlaySoundFile or PlayToRtp
        //if >=0 the message is related to a line function PlaySoundFile of line with this index
        PubCLMgrFirstDataReceived = 32         //first RTP data received on line, might be silence
        //lParam: line index;
    }

    public enum LineState
    {
        Inactive = 0,                        //line is inactive
        HookOffInternal = 1,                 //off hook, internal dialtone
        HookOffExternal = 2,                 //off hook, external dialtone
        Ringing = 3,                         //incoming call, ringing
        Dialing = 4,                         //outgoing call, we are dialing, no sound
        Alerting = 5,                        //outgoing call, alerting = ringing on destination
        Knocking = 6,                        //outgoing call, knocking = second call ringing on destination
        Busy = 7,                            //outgoing call, destination is busy
        Active = 8,                          //incoming / outgoing call, logical and physical connection is established
        OnHold = 9,                          //incoming / outgoing call, logical connection is established, destination gets music on hold
        ConferenceActive = 10,               //incoming / outgoing conference, logical and physical connection is established
        ConferenceOnHold = 11,               //incoming / outgoing conference, logical connection is established, not physcically connected
        Terminated = 12,                     //incoming / outgoing connection / call has been disconnected
        Transferring = 13,                   //special LSOnHold, call is awaiting to be transferred, peer gets special music on hold
        Disabled = 14                        //special LSInactive: wrap up time
    }

    public enum DisconnectReason
    {
        Normal = 0,
        Busy = 1,
        Rejected = 2,
        Cancelled = 3,
        Transferred = 4,
        JoinedConference = 5,
        NoAnswer = 6,
        TooLate = 7,
        DirectCallImpossible = 8,
        WrongNumber = 9,
        Unreachable = 10,
        CallDiverted = 11,
        CallRoutingFailed = 12,
        PermissionDenied = 13,
        NetworkCongestion = 14,
        NoChannelAvailable = 15,
        NumberChanged = 16,
        IncompatibleDestination = 17
    }
}



