/*---------------------------------------------------------------------------

   Swyx Solutions AG
   Joseph-von-Fraunhofer-Str. 13a
   44227 Dortmund

  ---------------------------------------------------------------------------

  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
  PURPOSE.

  Copyright (C) 2007 Swyx Solutions AG.

  All Rights Reserved.

----------------------------------------------------------------------------*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using CLMGRLib;

namespace Simple
{
    public partial class FrmMain : Form
    {
        //the ClientLineMgrClass is the top most object for the handling
        private ClientLineMgrClass pCLMgr = new ClientLineMgrClass();
        private ClientSdkEvents.ClientSdkEventSink MyEventSink = new ClientSdkEvents.ClientSdkEventSink();

        //here we define some ClientLines we want to handle.
        private ClientLine SelectedLine;
        private ClientLine[] Lines;

        public FrmMain()
        {
            InitializeComponent();
        }

        private void FrmMain_Load(object sender, EventArgs e)
        {
            Control.CheckForIllegalCrossThreadCalls = true;


            MyEventSink.Connect(pCLMgr, this, new ClientSdkEvents.LineManagerMessageHandler(OnLineManagerMessage));


            // get the Lines  from the Client
            int i;
            int LineCount;
            pCLMgr.GetLineCount(out LineCount);

            Lines = new ClientLine[LineCount];
            for (i = 0; i < LineCount; i++)
            {
                Lines[i] = (ClientLine)pCLMgr.DispGetLine(i);
            }

            ClientSdkEvents.LineState ls;
            string[] history;

            //Get the Linestate from Line1 and save it to the listview
            for (i = 0; i < LineCount; i++)
            {
                ls = (ClientSdkEvents.LineState)Lines[i].DispState;
                history = new string[4];
                history[0] = (i + 1).ToString();
                history[1] = ls.ToString();
                history[2] = "-";
                history[3] = DateTime.Now.ToLongTimeString();
                Add_NewHistoryItem(history);
            }
        }

        private void FrmMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            MyEventSink.Disconnect();
        }

        private void cmdDial_Click(object sender, EventArgs e)
        {
            //Just a Simple Dial of the Number or the Username on line 1 
            pCLMgr.SimpleDial(txtNumber.Text, 0);
        }

        private void cmdClear_Click(object sender, EventArgs e)
        {
            listview_history.Items.Clear();
        }

        //This function handles the Notification Events which come from the ClientLineManager
        private void OnLineManagerMessage(ClientSdkEvents.ClientSdkEventArgs e)
        {
            //get the selected Line
            SelectedLine = (ClientLine)pCLMgr.DispSelectedLine;

            //check what message we got
            switch (e.Msg)
            {

                case ClientSdkEvents.CLMgrMessage.CLMgrNumberOfLinesChangedMessage:
                    int linecount;
                    for (int i = 0; i < Lines.GetLength(0); i++)
                    {
                        if (Lines[i] != null)
                        {
                            Lines[i] = null;
                        }
                    }
                    pCLMgr.GetLineCount(out linecount);
                    if (linecount < 2)
                    {
                        cmd_Line2.Enabled = false;
                    }
                    else
                    {
                        cmd_Line2.Enabled = true;
                    }

                    if (linecount < 1)
                    {
                        cmd_Line1.Enabled = false;
                    }
                    else
                    {
                        cmd_Line1.Enabled = true;
                    }
                    Lines = new ClientLine[linecount];

                    for (int i = 0; i < linecount; i++)
                    {
                        Lines[i] = (ClientLine)pCLMgr.DispGetLine(i);
                    }

                    break;

                case ClientSdkEvents.CLMgrMessage.CLMgrClientShutDownRequest:
                    for (int i = 0; i < Lines.GetLength(0); i++)
                    {
                        if (Lines[i] != null)
                        {
                            Lines[i] = null;
                        }
                    }
                    break;
                // if the got CLMgrLineStateChangedMessageEx (int 28), we handle it 
                // as this is what we want
                case ClientSdkEvents.CLMgrMessage.CLMgrLineStateChangedMessageEx:

                    //Get Line
                    int line = e.Param & 0xff;
                    int high = e.Param >> 8;

                    ClientSdkEvents.LineState NewLineState;
                    //Get the LineState
                    NewLineState = (ClientSdkEvents.LineState)high;

                    //Write the event into the Listview
                    string[] history;
                    history = new string[4];
                    history[0] = (line + 1).ToString();
                    history[1] = NewLineState.ToString();
                    history[2] = "-";
                    history[3] = DateTime.Now.ToLongTimeString();

                    //Check which LineState we have (see enum for description)
                    switch (NewLineState)
                    {
                        case ClientSdkEvents.LineState.Terminated:
                            //if we have a Line terminated we want to know the reason
                            ClientSdkEvents.DisconnectReason m_DiscReason = (ClientSdkEvents.DisconnectReason)SelectedLine.DispDiscReason;
                            //enter the reason into the listview
                            history[2] = m_DiscReason.ToString();
                            break;
                    }
                    if ((int)NewLineState > -1)
                        Add_NewHistoryItem(history);
                    break;

                case ClientSdkEvents.CLMgrMessage.CLMgrLineDetailsChangedMessage:
                    {
                        IClientLinePub changedLinePub = (IClientLinePub)pCLMgr.DispGetLine(e.Param);
                        PubCLMgrLineDetails changedLineDetails;
                        changedLinePub.PubGetDetails(out changedLineDetails);
                        if (changedLineDetails.m_bIsInternalCall!=0)
                        {
                        }
                    }
                    break;

            }
        }

        private void Add_NewHistoryItem(string[] value)
        {
            ListViewItem HistoryItem;
            HistoryItem = new ListViewItem(value[0]);
            HistoryItem.SubItems.Add(value[1]);
            HistoryItem.SubItems.Add(value[2]);
            HistoryItem.SubItems.Add(value[3]);
            listview_history.Items.Add(HistoryItem);
        }
        //Click on Line Button 1
        private void cmd_Line1_Click(object sender, EventArgs e)
        {
            ClientSdkEvents.LineState ls;
            //we need the LineState of Line 1
            ls = (ClientSdkEvents.LineState)Lines[0].DispState;

            //handle the Line based on its current LineState
            switch (ls)
            {
                //if inactive and no number / name in the textfield just hook off
                case ClientSdkEvents.LineState.Inactive:
                    if (txtNumber.Text == "")
                    {
                        Lines[0].DispHookOff();
                    }
                    else // if we have text (number / name) then hookoff and dial
                    {
                        Lines[0].DispHookOff();
                        Lines[0].DispDial(txtNumber.Text);
                    }
                    break;
                // if we are off hook or dialing or alerting then hook on again
                case ClientSdkEvents.LineState.HookOffExternal:
                case ClientSdkEvents.LineState.HookOffInternal:
                case ClientSdkEvents.LineState.Dialing:
                case ClientSdkEvents.LineState.Alerting:
                    Lines[0].DispHookOn();
                    break;
                //if line is active set call on hold
                case ClientSdkEvents.LineState.Active:
                    Lines[0].DispHold();
                    break;

                //if line is on hold make the call active again
                case ClientSdkEvents.LineState.OnHold:
                    Lines[0].DispActivate();
                    break;
            }
        }
        //same as line1 but for line2
        private void cmd_Line2_Click(object sender, EventArgs e)
        {
            ClientSdkEvents.LineState ls;
            ls = (ClientSdkEvents.LineState)Lines[1].DispState;

            switch (ls)
            {
                case ClientSdkEvents.LineState.Inactive:
                    if (txtNumber.Text == "")
                    {
                        Lines[1].DispHookOff();
                    }
                    else
                    {
                        Lines[1].DispHookOff();
                        Lines[1].DispDial(txtNumber.Text);
                    }
                    break;

                case ClientSdkEvents.LineState.HookOffExternal:
                case ClientSdkEvents.LineState.HookOffInternal:
                case ClientSdkEvents.LineState.Dialing:
                case ClientSdkEvents.LineState.Alerting:
                    Lines[1].DispHookOn();
                    break;
                case ClientSdkEvents.LineState.Active:
                    Lines[1].DispHold();
                    break;
                case ClientSdkEvents.LineState.OnHold:
                    Lines[1].DispActivate();
                    break;
            }
        }

        // press the hook button to hook on 
        private void cmd_Hook_Click(object sender, EventArgs e)
        {
            ClientLine sl;
            sl = (ClientLine)pCLMgr.DispSelectedLine;
            sl.DispPressHook();
        }
    }

}



namespace ClientSdkEvents
{
    public class ClientSdkEventArgs : EventArgs
    {
        public CLMgrMessage Msg;
        public int Param;

        public ClientSdkEventArgs(CLMgrMessage msg, int param)
        {
            Msg = msg;
            Param = param;
        }
    }

    public delegate void LineManagerMessageHandler(ClientSdkEventArgs e);

    public class ClientSdkEventSink
    {
        private ClientLineMgrClass ConnectedLineManager;
        private IClientLineMgrEventsPub_PubOnLineMgrNotificationEventHandler EventHandler;
        private Form ConnectedForm;
        private LineManagerMessageHandler LineManagerMessageDelegateOfForm;

        public ClientSdkEventSink()
        {
            EventHandler = new IClientLineMgrEventsPub_PubOnLineMgrNotificationEventHandler(clmgr_EventSink);
        }

        public void Connect(ClientLineMgrClass lineManager, Form form, LineManagerMessageHandler lineManagerMessageDelegateOfForm)
        {
            ConnectedLineManager = lineManager;
            ConnectedForm = form;
            LineManagerMessageDelegateOfForm = lineManagerMessageDelegateOfForm;

            //add eventhandler for the PubOnlineMgrNotification Events
            ConnectedLineManager.PubOnLineMgrNotification += EventHandler;
        }

        public void Disconnect()
        {
            //remove eventhandler for the PubOnlineMgrNotification Events
            ConnectedLineManager.PubOnLineMgrNotification -= EventHandler;
            ConnectedLineManager = null;
            ConnectedForm = null;
            LineManagerMessageDelegateOfForm = null;
        }

        private void clmgr_EventSink(int msg, int param)
        {
            //this method receives the COM events from the client line manger
            if ((ConnectedForm != null) && (LineManagerMessageDelegateOfForm != null))
            {
                ConnectedForm.Invoke(LineManagerMessageDelegateOfForm, new ClientSdkEventArgs((CLMgrMessage)msg, param));
            }
        }
    }

    public enum CLMgrMessage
    {
        CLMgrLineStateChangedMessage = 0,      //state of at least one line has changed
        CLMgrLineSelectionChangedMessage = 1,  //line in focus has changed
        CLMgrLineDetailsChangedMessage = 2,    //details of at least one line have changed
        CLMgrCallDetailsMessage = 4,           //details of last call are available, post mortem for logging purpose
        CLMgrServerDownMessage = 5,            //server goes down, keep line manager, wait for ServerUp message
        CLMgrServerUpMessage = 6,              //server is up again, keep interfaces to line manger
        CLMgrWaveDeviceChanged = 7,            //speaker / micro has been switched on / off
        CLMgrGroupCallNotificationMessage = 8, //notification about group call
        CLMgrNumberOfLinesChangedMessage = 10, //the number of lines has changed
        CLMgrClientShutDownRequest = 11,       //Client Line Manager requests client to shutdown and release all interfaces
        CLMgrLineStateChangedMessageEx = 28,   //state of certain line has changed, lParam: LOWORD: line index of line that changed its state (starting with 0) HIWORD: new state of this line
        CLMgrSIPRegistrationStateChanged = 30, //registration state of SIP account has changed
        //lParam: LOBYTE: Account index
        //        HIBYTE: new state
        CLMgrWaveFilePlayed = 31,              //wave file playback finished
        //lParam: line index;
        //if -1, the message is related to a LineMgr function PlaySoundFile or PlayToRtp
        //if >=0 the message is related to a line function PlaySoundFile of line with this index
        PubCLMgrFirstDataReceived = 32         //first RTP data received on line, might be silence
        //lParam: line index;
    }

    public enum LineState
    {
        Inactive = 0,                        //line is inactive
        HookOffInternal = 1,                 //off hook, internal dialtone
        HookOffExternal = 2,                 //off hook, external dialtone
        Ringing = 3,                         //incoming call, ringing
        Dialing = 4,                         //outgoing call, we are dialing, no sound
        Alerting = 5,                        //outgoing call, alerting = ringing on destination
        Knocking = 6,                        //outgoing call, knocking = second call ringing on destination
        Busy = 7,                            //outgoing call, destination is busy
        Active = 8,                          //incoming / outgoing call, logical and physical connection is established
        OnHold = 9,                          //incoming / outgoing call, logical connection is established, destination gets music on hold
        ConferenceActive = 10,               //incoming / outgoing conference, logical and physical connection is established
        ConferenceOnHold = 11,               //incoming / outgoing conference, logical connection is established, not physcically connected
        Terminated = 12,                     //incoming / outgoing connection / call has been disconnected
        Transferring = 13,                   //special LSOnHold, call is awaiting to be transferred, peer gets special music on hold
        Disabled = 14                        //special LSInactive: wrap up time
    }

    public enum DisconnectReason
    {
        Normal = 0,
        Busy = 1,
        Rejected = 2,
        Cancelled = 3,
        Transferred = 4,
        JoinedConference = 5,
        NoAnswer = 6,
        TooLate = 7,
        DirectCallImpossible = 8,
        WrongNumber = 9,
        Unreachable = 10,
        CallDiverted = 11,
        CallRoutingFailed = 12,
        PermissionDenied = 13,
        NetworkCongestion = 14,
        NoChannelAvailable = 15,
        NumberChanged = 16,
        IncompatibleDestination = 17
    }
}



