'---------------------------------------------------------------------------
'
'   Swyx Solutions AG
'   Joseph-von-Fraunhofer-Str. 13a
'   44227 Dortmund
'
'  ---------------------------------------------------------------------------
'
'  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
'  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
'  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
'  PURPOSE.
'
'  Copyright (C) 2006 Swyx Solutions AG.
'
'  All Rights Reserved.
'
'  ---------------------------------------------------------------------------
'
'     Project:   Swyx Client SDK
'
'      Module:   Simple Dial Client sample
'
'      Author:   Bjoern Isemann
'
'  ---------------------------------------------------------------------------

Imports CLMGRLib

Public Class FrmMain

    'the ClientLineMgrClass is the top most object for the handling
    Private WithEvents pClmgr As New ClientLineMgrClass
    Private pClmgr1

    Delegate Sub LineManagerMessageHandler(ByVal msg As Integer, ByVal param As Integer)
    Public myDelegate As LineManagerMessageHandler


    'here we define some ClientLines we want to handle.
    Private SelectedLine As ClientLine
    Private Lines() As ClientLine
    Private HistoryItem As ListViewItem

    'This sub handles the Notification Events which come from the ClientLineManager
    Private Sub clmgr_PubOnLineMgrNotification(ByVal msg As Integer, ByVal param As Integer) Handles pClmgr.PubOnLineMgrNotification
        Me.Invoke(Me.myDelegate, New Object() {msg, param})
    End Sub


    'This sub handles the Notification Events which come from the ClientLineManager
    Public Sub MyLineManagerMessageHandler(ByVal msg As Integer, ByVal param As Integer)
        Dim i As Integer

        'converts the msg integer into a enum value
        Dim Message As CLMgrMessage = msg

        Try
            'we now dispatch the selected line from the clmgr
            SelectedLine = pClmgr.DispSelectedLine

            'check what message we got from the clmgr
            Select Case Message

                'do the number of lines changed? if yes release lines and bind new
                Case CLMgrMessage.CLMgrNumberOfLinesChangedMessage
                    Dim linecount As Integer
                    For i = 0 To Lines.GetUpperBound(0)
                        If Lines(i) IsNot Nothing Then
                            Lines(i) = Nothing
                        End If
                    Next
                    pClmgr.GetLineCount(linecount)
                    If linecount < 2 Then cmd_Line2.Enabled = False Else cmd_Line2.Enabled = True
                    If linecount < 1 Then cmd_Line1.Enabled = False Else cmd_Line1.Enabled = True
                    ReDim Lines(linecount)
                    For i = 0 To linecount - 1
                        Lines(i) = pClmgr.DispGetLine(i)
                    Next

                    'we look for shutdown to release the line pointer
                Case CLMgrMessage.CLMgrClientShutDownRequest
                    For i = 0 To Lines.GetUpperBound(0)
                        If Lines(i) IsNot Nothing Then
                            Lines(i) = Nothing
                        End If
                    Next

                    'CLMgrLineStateChangedMessageEx is the one we are interested in
                Case CLMgrMessage.CLMgrLineStateChangedMessageEx
                    'Get the line nr from the parameter
                    Dim line As Integer = LOWORD(param)

                    'get the linestate from the parameter
                    Dim NewLineState As LineState
                    NewLineState = HIWORD(param)

                    'make new listview entry
                    Dim history(4) As String
                    If NewLineState > -1 Then
                        history(0) = (line + 1).ToString
                        history(1) = NewLineState.ToString
                        history(2) = "-"
                        history(3) = Now.ToLongTimeString
                    End If

                    'here we check what linestate the line is in
                    'see the enum for description of each state
                    Select Case NewLineState
                        Case LineState.Active
                        Case LineState.Alerting
                        Case LineState.Busy
                        Case LineState.ConferenceActive
                        Case LineState.ConferenceActive
                        Case LineState.ConferenceOnHold
                        Case LineState.Dialing
                        Case LineState.Disabled
                        Case LineState.HookOffExternal
                        Case LineState.HookOffInternal
                        Case LineState.Inactive
                        Case LineState.Knocking
                        Case LineState.OnHold
                        Case LineState.Ringing
                        Case LineState.Terminated
                            'if the LineState is Terminated, ex when you hook on
                            'we want to know the disconnect reason
                            Dim m_DiscReason As DisconnectReason = SelectedLine.DispDiscReason
                            history(2) = m_DiscReason.ToString

                            'here we check for the disconnect reason
                            Select Case m_DiscReason
                                Case DisconnectReason.Busy
                                Case DisconnectReason.CallDiverted
                                Case DisconnectReason.CallRoutingFailed
                                Case DisconnectReason.Cancelled
                                Case DisconnectReason.DirectCallImpossible
                                Case DisconnectReason.IncompatibleDestination
                                Case DisconnectReason.JoinedConference
                                Case DisconnectReason.NetworkCongestion
                                Case DisconnectReason.NoAnswer
                                Case DisconnectReason.NoChannelAvailable
                                Case DisconnectReason.NumberChanged
                                Case DisconnectReason.PermissionDenied
                                Case DisconnectReason.Rejected
                                Case DisconnectReason.TooLate
                                Case DisconnectReason.Transferred
                                Case DisconnectReason.Unreachable
                                Case DisconnectReason.WrongNumber
                                Case DisconnectReason.Normal
                            End Select
                    End Select

                    'finaly add the new event item into the listview. 
                    Add_HistoryItem(history)
            End Select
        Catch ex As Exception
            'error handling here
        End Try
    End Sub

    'this function is just to add some information into the listview
    Private Sub Add_HistoryItem(ByVal value As String())
        HistoryItem = New ListViewItem(value(0))
        With HistoryItem
            .SubItems.Add(value(1))
            .SubItems.Add(value(2))
            .SubItems.Add(value(3))
        End With
        listview_history.Items.Add(HistoryItem)
    End Sub

    'gets the hiword of an integer
    Private Function HIWORD(ByVal b As Short) As Byte
        Dim h As Byte = b >> 8
        Return h
    End Function

    'gets the lowword of an integer
    Private Function LOWORD(ByVal b As Short) As Byte
        Dim l As Byte = b And &HFF
        Return l
    End Function

    'on form close we dispose the pclmgr
    Private Sub FrmMain_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        pClmgr = Nothing
    End Sub

    'if you press the simpledial button we just simple dial the number
    Private Sub cmdDial_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdDial.Click
        'Simple Dial the Number entered into txtnumber field 
        '0 is the line number (line 1 in this case as it is zero based)
        pClmgr.SimpleDial(txtNumber.Text, 0)
        txtNumber.Text = ""
    End Sub

    'on form load
    Private Sub FrmMain_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        'needed to prevent Thread exceptions which can occur as the clmgr.exe runs at a different thread id
        Control.CheckForIllegalCrossThreadCalls = True

        'get the lines
        Dim linecount, i As Integer
        pClmgr.GetLineCount(linecount)
        ReDim Lines(linecount - 1)
        For i = 0 To linecount - 1
            Lines(i) = pClmgr.DispGetLine(i)
        Next

        Dim history(4) As String

        'and we get the linestates of these lines for information
        Dim ls As LineState
        For i = 0 To Lines.GetUpperBound(0)
            ls = Lines(0).DispState()
            history(0) = (i + 1).ToString
            history(1) = ls.ToString
            history(2) = "-"
            history(3) = Now.ToLongTimeString
            Add_HistoryItem(history)
        Next
        'add these linestates information into the history


        myDelegate = New LineManagerMessageHandler(AddressOf MyLineManagerMessageHandler)

        pClmgr1 = New ClientLineMgrClass


    End Sub

    'clear the history
    Private Sub cmdClear_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdClear.Click
        listview_history.Items.Clear()
    End Sub

    'if you click the line 1 button
    Private Sub cmdLine1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmd_Line1.Click
        Dim ls As LineState

        'get the linestate of this line
        ls = Lines(0).DispState

        'handle the line different based on the current state
        Select Case ls
            'if the line is inactive
            Case LineState.Inactive
                'and the number in the text element is empty
                If txtNumber.Text = "" Then
                    'hook off
                    Lines(0).DispHookOff()
                Else
                    'we have a number in the textfield
                    'hook off and and dial this number
                    Lines(0).DispHookOff()
                    Lines(0).DispDial(txtNumber.Text)
                End If

                'if we have the states hookoff, dialing or alerting
            Case LineState.HookOffExternal, LineState.HookOffInternal, LineState.Dialing, LineState.Alerting
                'hook on
                Lines(0).DispHookOn()
                'if the line is active (a call is active)
            Case LineState.Active
                'set the call on hold
                Lines(0).DispHold()
                'if the line (call) is on hold
            Case LineState.OnHold
                'reactive this call
                Lines(0).DispActivate()
        End Select
    End Sub

    'hook button
    Private Sub cmd_Hook_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmd_Hook.Click
        Dim sl As ClientLine
        'get the current selected line
        sl = pClmgr.DispSelectedLine()
        'press the hook button, so hook on
        sl.DispPressHook()
    End Sub

    Private Sub cmd_Line2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmd_Line2.Click
        Dim ls As LineState

        'get the linestate of this line
        ls = Lines(1).DispState

        'handle the line different based on the current state
        Select Case ls
            'if the line is inactive
            Case LineState.Inactive
                'and the number in the text element is empty
                If txtNumber.Text = "" Then
                    'hook off
                    Lines(1).DispHookOff()
                Else
                    'we have a number in the textfield
                    'hook off and and dial this number
                    Lines(1).DispHookOff()
                    Lines(1).DispDial(txtNumber.Text)
                End If

                'if we have the states hookoff, dialing or alerting
            Case LineState.HookOffExternal, LineState.HookOffInternal, LineState.Dialing, LineState.Alerting
                'hook on
                Lines(1).DispHookOn()
                'if the line is active (a call is active)
            Case LineState.Active
                'set the call on hold
                Lines(1).DispHold()
                'if the line (call) is on hold
            Case LineState.OnHold
                'reactive this call
                Lines(1).DispActivate()
        End Select
    End Sub
End Class

Public Enum CLMgrMessage
    CLMgrLineStateChangedMessage = 0      'state of at least one line has changed
    CLMgrLineSelectionChangedMessage = 1 'line in focus has changed
    CLMgrLineDetailsChangedMessage = 2    'details of at least one line have changed
    CLMgrCallDetailsMessage = 4           'details of last call are available, post mortem for logging purpose
    CLMgrServerDownMessage = 5            'server goes down, keep line manager, wait for ServerUp message
    CLMgrServerUpMessage = 6              'server is up again, keep interfaces to line manger
    CLMgrWaveDeviceChanged = 7            'speaker / micro has been switched on / off
    CLMgrGroupCallNotificationMessage = 8 'notification about group call
    CLMgrNumberOfLinesChangedMessage = 10 'the number of lines has changed
    CLMgrClientShutDownRequest = 11       'Client Line Manager requests client to shutdown and release all interfaces
    CLMgrLineStateChangedMessageEx = 28   'state of certain line has changed, lParam: LOWORD: line index of line that changed its state (starting with 0) HIWORD: new state of this line
End Enum

Public Enum LineState
    Inactive = 0                        'line is inactive
    HookOffInternal = 1                 'off hook, internal dialtone
    HookOffExternal = 2                 'off hook, external dialtone
    Ringing = 3                         'incoming call, ringing
    Dialing = 4                         'outgoing call, we are dialing, no sound
    Alerting = 5                        'outgoing call, alerting = ringing on destination
    Knocking = 6                        'outgoing call, knocking = second call ringing on destination
    Busy = 7                            'outgoing call, destination is busy
    Active = 8                          'incoming / outgoing call, logical and physical connection is established
    OnHold = 9                          'incoming / outgoing call, logical connection is established, destination gets music on hold
    ConferenceActive = 10               'incoming / outgoing conference, logical and physical connection is established
    ConferenceOnHold = 11               'incoming / outgoing conference, logical connection is established, not physcically connected
    Terminated = 12                     'incoming / outgoing connection / call has been disconnected
    Transferring = 13                   'special LSOnHold, call is awaiting to be transferred, peer gets special music on hold
    Disabled = 14                       'special LSInactive: wrap up time
End Enum

Public Enum DisconnectReason
    Normal = 0
    Busy = 1
    Rejected = 2
    Cancelled = 3
    Transferred = 4
    JoinedConference = 5
    NoAnswer = 6
    TooLate = 7
    DirectCallImpossible = 8
    WrongNumber = 9
    Unreachable = 10
    CallDiverted = 11
    CallRoutingFailed = 12
    PermissionDenied = 13
    NetworkCongestion = 14
    NoChannelAvailable = 15
    NumberChanged = 16
    IncompatibleDestination = 17
End Enum



