/*---------------------------------------------------------------------------

   Swyx Solutions AG
   Joseph-von-Fraunhofer-Str. 13a
   44227 Dortmund

  ---------------------------------------------------------------------------

  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
  PURPOSE.

  Copyright (C) 2007 Swyx Solutions AG.

  All Rights Reserved.

----------------------------------------------------------------------------*/

// MyResolver.cpp : Implementation of CMyResolver
#include "stdafx.h"
#include "PlugInSample.h"
#include "..\..\..\PublicInterface\CLMgrPubTypes.h"
#include "MyResolver.h"
#include "githelp.h"
#include <comdef.h>



_ATL_FUNC_INFO DispOnLineMgrNotificationInfo={CC_STDCALL, VT_EMPTY, 2, {VT_I4, VT_I4} };



/////////////////////////////////////////////////////////////////////////////
// CMyResolver
CMyResolver::CMyResolver()
{
    m_dwCLMgrCookie=0;
}



CMyResolver::~CMyResolver()
{
    if(m_dwCLMgrCookie)
    {
        CoRevokeInterfaceFromGlobal(m_dwCLMgrCookie);
        m_dwCLMgrCookie=0;
    }
}



//IClientResolverAddIn

/*
 * This method will resolve a phone number to a name
 *
 * Input:  Number
 * Output: Name.
 * If the number can be resolved, *pName is the resolved name and the HRESULT is S_OK.
 * If the number cannot be resolved, *pName is set to NULL and the HRESULT is S_FALSE.
 * If Number is NULL or pName is NULL, HRESULT will be E_INVALIDARG
 */
STDMETHODIMP CMyResolver::ResolveNumber(/*[in]*/ BSTR Number, /*[out]*/ BSTR *pName)
{
    HRESULT         hRetVal=S_FALSE;
    CString         sNumber=(char*)_bstr_t(Number);
    CString         ResolvedName="Dilbert";

    if( (Number!=NULL) && (pName!=NULL) )
    {
        *pName=ResolvedName.AllocSysString();
        hRetVal=S_OK;
    }
    else
    {
        hRetVal=E_INVALIDARG;
    }

    return hRetVal;
}



/*
 * This method will return the preferred phone number style to be resolved
 *
 *     typedef enum
 *     {
 *         PubCLMgrNumberStyleFull,        //e.g. 0004923147770 or 0023147770
 *         PubCLMgrNumberStylePlain,       //e.g. 4923147770 or 23147770
 *         PubCLMgrNumberStyleCanonical    //e.g. +49 (231) 47770 or +49 (231) 4777-0
 *     } PubCLMgrNumberStyle;
 *
 */
STDMETHODIMP CMyResolver::GetPreferredNumberStyle(/*[out]*/ DWORD *pStyle)
{
    HRESULT hr=S_OK;

    if(pStyle)
    {
        *pStyle=PubCLMgrNumberStyleFull;
    }
    else
    {
        hr=E_INVALIDARG;
    }

    return hr;
}



//IClientAddInLoader

/*
 * This method will be called after loading the PlugIn
 *
 * Input:  dwReserved, must be NULL
 * If HRESULT is not S_OK, the PlugIn will be released, UnInitialize will not be called in that case
 */
STDMETHODIMP CMyResolver::Initialize(/*[in]*/ DWORD dwReserved)
{
    HRESULT hRetVal=S_OK;
    HRESULT hr;
    IClientLineMgrPub *pIClientLineMgrPub=NULL;

    //create line manager object on local machine
    hr=CoCreateInstance(CLSID_ClientLineMgr, NULL, CLSCTX_LOCAL_SERVER, IID_IClientLineMgrPub,(void**)&pIClientLineMgrPub);

    if(SUCCEEDED(hr))
    {
        hr=CoRegisterInterfaceInGlobal(pIClientLineMgrPub, IID_IClientLineMgrPub, &m_dwCLMgrCookie);
        if(FAILED(hr))
        {
            m_dwCLMgrCookie=0;
            hRetVal=E_FAIL;
        }

        //register our event sink for events from line manager
        hr=IDispEventImpl<1, CMyResolver, &DIID_IClientLineMgrEventsDisp, &LIBID_CLMGRLib>::DispEventAdvise(pIClientLineMgrPub);

        pIClientLineMgrPub->Release();
    }

    if(FAILED(hRetVal) && m_dwCLMgrCookie)
    {
        CoRevokeInterfaceFromGlobal(m_dwCLMgrCookie);
        m_dwCLMgrCookie=0;
    }

    return hRetVal;
}



/*
 * This method will return the visible name of the PlugIn
 *
 */
STDMETHODIMP CMyResolver::GetName(/*[out]*/ BSTR *pName)
{
    HRESULT hr=S_OK;

    if(pName)
    {
        CString sName;

        sName.LoadString(IDR_MAINFRAME);

        if(sName.IsEmpty())
        {
            sName="Unknown";
        }

        *pName=sName.AllocSysString();
    }
    else
    {
        hr=E_INVALIDARG;
    }

    return hr;
}



/*
 * This method will return the version of the PlugIn
 * Format: e.g. "3.2.0.4"   for Version 3.20 build 4
 */
STDMETHODIMP CMyResolver::GetVersion(/*[out]*/ BSTR *pVersion)
{
    HRESULT hr=S_OK;

    if(pVersion)
    {
        CString sName="Unknown";

        *pVersion=sName.AllocSysString();
    }
    else
    {
        hr=E_INVALIDARG;
    }

    return hr;
}



/*
 * This method will be called before releasing the PlugIn
 *
 * Input:  dwReserved, must be NULL
 */
STDMETHODIMP CMyResolver::UnInitialize(/*[in]*/ DWORD dwReserved)
{
    IClientLineMgrPub *pIClientLineMgrPub=NULL;

    if(m_dwCLMgrCookie)
    {
        if(SUCCEEDED(CoGetInterfaceFromGlobal(m_dwCLMgrCookie, IID_IClientLineMgrPub, (void**)&pIClientLineMgrPub)))
        {
            //unregister our event sink from line manager events
            HRESULT hr=IDispEventImpl<1, CMyResolver, &DIID_IClientLineMgrEventsDisp, &LIBID_CLMGRLib>::DispEventUnadvise(pIClientLineMgrPub);

            pIClientLineMgrPub->Release();
        }

        CoRevokeInterfaceFromGlobal(m_dwCLMgrCookie);
        m_dwCLMgrCookie=0;
    }

    return S_OK;
}



//state of a line has changed
void CMyResolver::OnLineStateChanged(int iLine)
{
    HRESULT             hr;
    IClientLineMgrPub   *pIClientLineMgrPub=NULL;
    IClientLinePub      *pILine=NULL;

    if(m_dwCLMgrCookie)
    {
        if(SUCCEEDED(CoGetInterfaceFromGlobal(m_dwCLMgrCookie, IID_IClientLineMgrPub, (void**)&pIClientLineMgrPub)))
        {
            hr=pIClientLineMgrPub->PubGetLine(iLine, &pILine);

            if(SUCCEEDED(hr) && pILine)
            {
                long lState=PubLSNone;

                hr=pILine->PubGetState(&lState);

                switch(lState)
                {
                    case PubLSRinging:
                        {
                            //pop up / create contact etc.
                            PubCLMgrLineDetails Details;
                            BOOL                bInternalCall=FALSE;
                            CString             PeerName;
                            CString             PeerNumber;

                            hr=pILine->PubGetDetails(&Details);

                            PeerName=(char*)_bstr_t(Details.m_sPeerName);
                            PeerNumber=(char*)_bstr_t(Details.m_sPeerNumber);
                        



                            SysFreeString(Details.m_sAcknowledgedDialstring);
                            SysFreeString(Details.m_sPeerNumber);
                            SysFreeString(Details.m_sPeerName);
                            SysFreeString(Details.m_sCalledExtension);
                            SysFreeString(Details.m_sCalledName);
                            SysFreeString(Details.m_sRedirectedFromNumber);
                            SysFreeString(Details.m_sRedirectedFromName);
                            SysFreeString(Details.m_sBlindCallTransferSourceNumber);
                            SysFreeString(Details.m_sBlindCallTransferSourceName);
                            SysFreeString(Details.m_sChargingDetails);
                        }
                        break;

                    case PubLSTerminated:
                    case PubLSBusy:
                    case PubLSInactive:
                    case PubLSHookOffInternal:
                    case PubLSHookOffExternal:
                    case PubLSDialing:
                    case PubLSAlerting:
                    case PubLSKnocking:
                    case PubLSActive:
                    case PubLSOnHold:
                    case PubLSConferenceActive:
                    case PubLSConferenceOnHold:
                    case PubLSTransferring:
                    case PubLSDisabled:
                    default:
                        break;
                }

                pILine->Release();
            }

            pIClientLineMgrPub->Release();
        }
    }
}



//details of last call are available, after disconnect, for logging purpose
void CMyResolver::OnLastCallDetails(int iLine)
{
    HRESULT             hr;
    IClientLineMgrPub   *pIClientLineMgrPub=NULL;
    IClientLinePub      *pILine=NULL;



    if(m_dwCLMgrCookie)
    {
        if(SUCCEEDED(CoGetInterfaceFromGlobal(m_dwCLMgrCookie, IID_IClientLineMgrPub, (void**)&pIClientLineMgrPub)))
        {
            hr=pIClientLineMgrPub->PubGetLine(iLine, &pILine);

            if(SUCCEEDED(hr) && pILine)
            {
                PubCLMgrCallDetails LastCallDetails;

                if(SUCCEEDED(pILine->PubGetLastCallDetails(&LastCallDetails)))
                {
                    //pop up / create contact etc.

                    CTime StartTime=CTime(  LastCallDetails.m_LastCallStartTime.year,
                                            LastCallDetails.m_LastCallStartTime.month,
                                            LastCallDetails.m_LastCallStartTime.day,
                                            LastCallDetails.m_LastCallStartTime.hour,
                                            LastCallDetails.m_LastCallStartTime.minute,
                                            LastCallDetails.m_LastCallStartTime.second);

                    CTime EndTime=CTime(    LastCallDetails.m_LastCallFinishedTime.year,
                                            LastCallDetails.m_LastCallFinishedTime.month,
                                            LastCallDetails.m_LastCallFinishedTime.day,
                                            LastCallDetails.m_LastCallFinishedTime.hour,
                                            LastCallDetails.m_LastCallFinishedTime.minute,
                                            LastCallDetails.m_LastCallFinishedTime.second);

                    CTimeSpan Duration=EndTime-StartTime;

                    CString PeerName=(char*)_bstr_t(LastCallDetails.m_sLastCallPeersPbxUserName);
                    CString PeerNumber=(char*)_bstr_t(LastCallDetails.m_sLastCallPeersNumber);

                    CString Charging=(char*)_bstr_t(LastCallDetails.m_sLastCallChargingDetails);



                    SysFreeString(LastCallDetails.m_sLastCallLcrPrefix);
                    SysFreeString(LastCallDetails.m_sLastCallOriginatingExtension);
                    SysFreeString(LastCallDetails.m_sLastCallChargingDetails);
                    SysFreeString(LastCallDetails.m_sLastCallPeersPbxUserName);
                    SysFreeString(LastCallDetails.m_sLastCallPeersNumber);
                }

                pILine->Release();
            }

            pIClientLineMgrPub->Release();
        }
    }
}



/*
 * signal line manager message to client
 * msg:   same as wParam send with WM_LineMgrNotification -> CLMgrMessages
 * param: same as lParam send with WM_LineMgrNotification -> CLMgrMessages
 */
void __stdcall CMyResolver::DispOnLineMgrNotification(/*[in]*/ long msg, /*[in]*/ long param)
{
    switch(msg)
    {
        case PubCLMgrLineStateChangedMessage:
            //state of a line has changed. lParam: index of line
            OnLineStateChanged(param);
            break;

        case PubCLMgrCallDetailsMessage:
            //details of last call are available, after disconnect, for logging purpose
            //lParam: index of line           
            OnLastCallDetails(param);
            break;

        case PubCLMgrNumberOfLinesChangedMessage:
        case PubCLMgrClientShutDownRequest:
        case PubCLMgrLineDetailsChangedMessage:
        case PubCLMgrLineSelectionChangedMessage:
        case PubCLMgrHandsetStateChangedMessage:
        case PubCLMgrNameKeyStateChangedMessage:
        case PubCLMgrUserDataChangedMessage:
        case PubCLMgrGroupCallNotificationMessage:
        case PubCLMgrServerDownMessage:
        case PubCLMgrServerUpMessage:
        case PubCLMgrWaveDeviceChanged:
        case PubCLMgrPowerSuspendMessage:
        case PubCLMgrPowerResumeMessage:
        default:
            break;
    }
}

