/*---------------------------------------------------------------------------

   Swyx Solutions AG
   Joseph-von-Fraunhofer-Str. 13a
   44227 Dortmund

  ---------------------------------------------------------------------------

  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
  PURPOSE.

  Copyright (C) 2007 Swyx Solutions AG.

  All Rights Reserved.

----------------------------------------------------------------------------*/




// MyResolver.h : Declaration of the CMyResolver

#ifndef __MYRESOLVER_H_
#define __MYRESOLVER_H_

#include "resource.h"       // main symbols



extern _ATL_FUNC_INFO DispOnLineMgrNotificationInfo;


/////////////////////////////////////////////////////////////////////////////
// CMyResolver
class ATL_NO_VTABLE CMyResolver : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CMyResolver, &CLSID_MyResolver>,
	public IDispEventImpl<1, CMyResolver, &DIID_IClientLineMgrEventsDisp, &LIBID_CLMGRLib>,
    public IClientAddInLoader,
    public IClientResolverAddIn
{
public:
	CMyResolver();
	virtual ~CMyResolver();


    DWORD           m_dwCLMgrCookie;


DECLARE_REGISTRY_RESOURCEID(IDR_MYRESOLVER)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CMyResolver)
	COM_INTERFACE_ENTRY(IClientAddInLoader)
	COM_INTERFACE_ENTRY(IClientResolverAddIn)   
END_COM_MAP()


// Line Manager Events
BEGIN_SINK_MAP(CMyResolver)
   SINK_ENTRY_INFO(1, DIID_IClientLineMgrEventsDisp, 0x00000001, DispOnLineMgrNotification, &DispOnLineMgrNotificationInfo)
END_SINK_MAP()


//IClientResolverAddIn
public:
    /*
     * This method will resolve a phone number to a name
     *
     * Input:  Number
     * Output: Name.
     * If the number can be resolved, *pName is the resolved name and the HRESULT is S_OK.
     * If the number cannot be resolved, *pName is set to NULL and the HRESULT is S_FALSE.
     * If Number is NULL or pName is NULL, HRESULT will be E_INVALIDARG
     */
	STDMETHOD(ResolveNumber)(/*[in]*/ BSTR Number, /*[out]*/ BSTR *pName);

    /*
     * This method will return the preferred phone number style to be resolved
     *
     *     typedef enum
     *     {
     *         PubCLMgrNumberStyleFull,        //e.g. 0004923147770 or 0023147770
     *         PubCLMgrNumberStylePlain,       //e.g. 4923147770 or 23147770
     *         PubCLMgrNumberStyleCanonical    //e.g. +49 (231) 47770 or +49 (231) 4777-0
     *     } PubCLMgrNumberStyle;
     *
     */
	STDMETHOD(GetPreferredNumberStyle)(/*[out]*/ DWORD *pStyle);


//IClientAddInLoader
public:
    /*
     * This method will be called after loading the PlugIn
     *
     * Input:  dwReserved, must be NULL
     * If HRESULT is not S_OK, the PlugIn will be released, UnInitialize will not be called in that case
     */
	STDMETHOD(Initialize)(/*[in]*/ DWORD dwReserved);

    /*
     * This method will return the visible name of the PlugIn
     *
     */
	STDMETHOD(GetName)(/*[out]*/ BSTR *pName);

    /*
     * This method will return the version of the PlugIn
     * Format: e.g. "3.2.0.4"   for Version 3.20 build 4
     */
	STDMETHOD(GetVersion)(/*[out]*/ BSTR *pVersion);

    /*
     * This method will be called before releasing the PlugIn
     *
     * Input:  dwReserved, must be NULL
     */
	STDMETHOD(UnInitialize)(/*[in]*/ DWORD dwReserved);


//IClientLineMgrEventsDisp
public:
    /*
     * signal line manager message to client
     * msg:   same as wParam send with WM_LineMgrNotification -> CLMgrMessages
     * param: same as lParam send with WM_LineMgrNotification -> CLMgrMessages
     */
    void __stdcall DispOnLineMgrNotification(/*[in]*/ long msg, /*[in]*/ long param);
    
    
public:

    //state of a line has changed
    void OnLineStateChanged(int iLine);
    
    //details of last call are available, after disconnect, for logging purpose
    void OnLastCallDetails(int iLine);


};

#endif //__MYRESOLVER_H_
